/*
* term.c                                                    Version 5.2.0
*
* Terminal I/O routines for STM32.
*
* Copyright (c) 1998-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Authors: David Moore, Yingbo Hu, and Xingsheng Wan
*
*****************************************************************************/

#include "smxbase.h"
#include "bsp.h"

#if SB_CON_OUT && !SB_CFG_UARTI

int sur_OutByte(uint port, u8 b, SUR_CALLBACK cb, uint tmo)
{
   (void)port;
   (void)cb;
   (void)tmo;

   while(!txReady(SB_CON_OUT_PORT)) {}
   UartPutChar(SB_CON_OUT_PORT, b);
   return 1;
}

int sur_OutData(uint port, u8 *psrc, uint len, u8 term, SUR_CALLBACK cb, uint tmo)
{
   uint i;
   (void)port;
   (void)cb;
   (void)tmo;

   if(len != 0)
   {
      for (i = 0; i < len; i++)
      {
         while(!txReady(SB_CON_OUT_PORT)) {}
         UartPutChar(SB_CON_OUT_PORT, psrc[i]);
      }
      return len;
   }
   else
   {
      for (i = 0; psrc[i] != term; i++)
      {
         while(!txReady(SB_CON_OUT_PORT)) {}
         UartPutChar(SB_CON_OUT_PORT, psrc[i]);
      }
      return i;
   }
}
#endif /* SB_CON_OUT && !SB_CFG_UARTI */

#if SB_CON_IN

void sb_KbdInit(void)
{
}

void sb_KbdExit(void)
{
}

char sb_KbdGetCharPoll(void)
{
#if SB_CFG_UARTI
   char key;
   if(1 == sur_InByte(SB_CON_IN_PORT, (u8 *)&key, NULL, 0))
      return key;
#else
   if (rxReady(SB_CON_IN_PORT))
      return (u8)UartGetChar(SB_CON_IN_PORT);
#endif
   return 0;
}

#endif /* SB_CON_IN */

