/*
* bbsp.h                                                    Version 5.2.0
*
* smx Board Support Package API header file.
*
* Include in all C files that use smx. These are the API call prototypes
* and definitions for functions that must be supplied in the board support
* package. See documentation in BSP API section of smxBase User's Guide.
*
* Copyright (c) 2001-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore
*
*****************************************************************************/

#ifndef SB_BBSP_H
#define SB_BBSP_H

/********** Defines **********/

#ifdef __cplusplus
extern "C" {
#endif

/********** Global Externs **********/

extern BOOLEAN   sb_tick_init_done;       /* sb_TickInit() sets */
extern const u32 sb_ticktmr_clkhz;        /* tick timer/counter clock hz (input clock frequency divided by any prescalers) */
extern const u32 sb_ticktmr_cntpt;        /* tick timer/counter counts per tick */
extern u32       sb_tmo;                  /* time measurement overhead */

/********** BSP API **********/

/* Interrupt Handling Functions and Shorthand Macros */

BOOLEAN  sb_IntCtrlInit(void);
void     sb_IntStateRestore(CPU_FL prev_state);
CPU_FL   sb_IntStateSaveDisable(void);
BOOLEAN  sb_IntTrapVectSet(int int_num, ISR_PTR isr_ptr);
ISR_PTR  sb_IntVectGet(int int_num, u32* extra_info);
BOOLEAN  sb_IntVectSet(int int_num, ISR_PTR isr_ptr);
BOOLEAN  sb_IRQClear(int irq_num);
BOOLEAN  sb_IRQConfig(int irq_num);
BOOLEAN  sb_IRQEnd(int irq_num);
BOOLEAN  sb_IRQMask(int irq_num);
BOOLEAN  sb_IRQUnmask(int irq_num);
BOOLEAN  sb_IRQsMask(void);
BOOLEAN  sb_IRQsUnmask(void);
int      sb_IRQToInt(int irq_num);
ISR_PTR  sb_IRQVectGet(int irq_num, u32* extra_info);
BOOLEAN  sb_IRQVectSet(int irq_num, ISR_PTR isr_ptr);
BOOLEAN  sb_ISRInstall(int irq_num, uint par, ISRC_PTR fun, const char* name);
BOOLEAN  sb_ISRRestore(int irq_num);

#define  sb_INT_DISABLE_S(s)  s = sb_IntStateSaveDisable()
#define  sb_INT_ENABLE_R(s)   sb_IntStateRestore(s)

/* Memory Functions */

void*    sb_DMABufferAlloc(uint num_bytes);
BOOLEAN  sb_DMABufferFree(void* buf);

/* Time Functions */

BOOLEAN  sb_ClocksInit(void);
#if defined(SB_CPU_BLACKFIN)
void     sb_DelayMsec(u32 num);
#else
#define  sb_DelayMsec(num) sb_DelayUsec(1000*(num))
#endif
void     sb_DelayUsec(u32 num);
u32      sb_PtimeGet(void);
BOOLEAN  sb_StimeSet(void);
BOOLEAN  sb_TickInit(void);
BOOLEAN  sb_TickIntEnable(void);

/* Misc Functions */

BOOLEAN  sb_ConsoleInInit(void);
BOOLEAN  sb_ConsoleOutInit(void);
void     sb_DemoExit(void);
void     sb_DemoInit(void);
BOOLEAN  sb_EVBInit(void);
void     sb_Exit(int retcode);
BOOLEAN  sb_PeripheralsInit(void);
void     sb_Reboot(void);
void     sb_Restart(void);

/********** Optional API **********/

void     sb_HWInitAtMain(void);

/* LCD / OLED */

void     sb_LCDInit(void);
void     sb_LCDWriteString(const char* str, uint line, u32 color, u32 attr);

/* LED */

void     sb_LEDInit(void);
void     sb_LEDWrite7Seg(u32 val);
void     sb_LEDWrite7SegNum(int num);
void     sb_LEDWriteRow(u32 val);

#ifdef __cplusplus
}
#endif

#endif /* SB_BBSP_H */

