/*
* svc.c                                                     Version 5.2.0
*
* ARM-M SVC System call shell functions
*
* Copyright (c) 2016-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* SMX is protected by patents listed in smx.h.
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: Ralph Moore
*
*****************************************************************************/

#include "xsmx.h"
#include "svc.h"

#if SMX_CFG_MPU

/*===========================================================================*
*                         System Service Jump Table <1>                      *
*===========================================================================*/

/* system service table indices */
enum ssndx {LIM, AS, ASL, HF, HM, MUCR, MUG, MUR, ST, SEG, SSG, IRQM, IRQU, PK, 
            PTMG, EM, END};

u32 smx_sst[] = {
   (u32)END,
   (u32)smx_SchedAutoStop,
   (u32)smx_SchedAutoStopLSR,
   (u32)smx_HeapFree,
   (u32)smx_HeapMalloc,
   (u32)smx_MutexCreate,
   (u32)smx_MutexGet,
   (u32)smx_MutexRel,
   (u32)smx_SemTest,
   (u32)smx_SysEtimeGet,
   (u32)smx_SysStimeGet,
   (u32)sb_IRQMask,
   (u32)sb_IRQUnmask,
   (u32)sb_Peek,
   (u32)sb_PtimeGet,
   (u32)smx_EM,
};

/*===========================================================================*
*                                uSSR Shells                                 *
*===========================================================================*/

#include "xapiu.h"

#pragma default_function_attributes = @ ".svc_text"

NI void smxu_SchedAutoStop(void)
{
   sb_SVC(AS)
}

NI void smxu_SchedAutoStopLSR(void)
{
   sb_SVC(ASL)
}

NI BOOLEAN smxu_HeapFree(void *bp, u32 hn)
{
   sb_SVCH(HF)
}

NI void* smxu_HeapMalloc(u32 sz, u32 an, u32 hn)
{
   sb_SVCH(HM)
}

NI MUCB_PTR smxu_MutexCreate(u8 pi, u8 ceiling, const char *name, MUCB_PTR* muhp)
{
   sb_SVC(MUCR)
}

NI BOOLEAN smxu_MutexGet(MUCB_PTR mtx, u32 timeout)
{
   sb_SVC(MUG)
}

NI BOOLEAN smxu_MutexRel(MUCB_PTR mtx)
{
   sb_SVC(MUR)
}

NI BOOLEAN smxu_SemTest(SCB_PTR sem, u32 timeout)
{
   sb_SVC(ST)
}

NI u32 smxu_SysEtimeGet(u32 flags)
{
   sb_SVC(SEG)
}

NI u32 smxu_SysStimeGet(void)
{
   sb_SVC(SSG)
}

NI BOOLEAN sbu_IRQMask(int irq_num)
{
   sb_SVC(IRQM)
}

NI BOOLEAN sbu_IRQUnmask(int irq_num)
{
   sb_SVC(IRQU)
}

NI u32 sbu_Peek(SB_PK_PAR par)
{
   sb_SVC(PK)
}

NI u32 sbu_PtimeGet(void)
{
   sb_SVC(PTMG)
}

NI void smxu_EM(SMX_ERRNO errnum, void *h)
{
   sb_SVC(EM)
}

/* 
   Notes:
   1. When adding new services:
      a. Add unique acronym to ssndx enum, wherever convenient.
      b. Put function name into smx_sst[] in same place as ssndx.
      c. Add smxu_ shell function using above acronym.
      d. Put function prototype into xapiu.h followed by mapping macro.
      f. Add #undef for mapping macro to xapip.h.
   2. smx_IFEM is repeated by the smx_SVC_Handler in xmpu_xxx.s for all  
      indices > SMX_SST_LAST
   3. smxu_Heap calls assume that the heap mutex is free. If it is, the call
      executes and returns. If not, and if no timeout is specified, the call 
      fails and returns 0. If a timeout is specified, the current task is 
      suspended on the heap mutex. If the timeout elapses, the call fails and
      returns 0. If the mutex is gained, -2 is returned and a recall executes 
      and returns. (-2 is used because -1 is a valid return for some heap 
      functions.)
*/
#endif /* SMX_CFG_MPU */
