/*
* smxaware.c                                                Version 5.2.0
*
* smxAware init, print ring, and utilities.
* smxAware Live init, server task, and subroutines.
*
* Copyright (c) 1999-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: Marty Cochran
*
*****************************************************************************/

#if defined(SMXAWARE)  /* condition out whole file if SMXAWARE is not enabled */

#include "smxbase.h"
#include "smxaware.h"
#if defined(SMX_TSMX)
#include "tmain.h"  /* for smx_rtcb */
#else
#include "smxmain.h"  /* for smx_rtcb */
#endif

#pragma section = "EB"
#pragma section = "EVB"

/* Configuration Table */
struct
{
   u32   lq_size        = SMX_CFG_LQ_SIZE;          /* KEEP FIRST <1> */
   u32   sec            = SMX_CFG_TICKS_PER_SEC;
   u32   num_blocks     = SMX_CFG_NUM_BLOCKS;
   u32   num_egs        = SMX_CFG_NUM_EVGS;
   u32   num_eqs        = SMX_CFG_NUM_EQS;
   u32   num_msgs       = SMX_CFG_NUM_MSGS;
   u32   num_mtxs       = SMX_CFG_NUM_MTXS;
   u32   num_pipes      = SMX_CFG_NUM_PIPES;
   u32   num_pools      = SMX_CFG_NUM_POOLS;
   u32   num_sems       = SMX_CFG_NUM_SEMS;
   u32   num_tasks      = SMX_CFG_NUM_TASKS;
   u32   num_tmrs       = SMX_CFG_NUM_TIMERS;
   u32   num_xchgs      = SMX_CFG_NUM_XCHGS;
   u32   heap_address   = SMX_CFG_HEAP_ADDRESS;
   u32   heap_space     = SMX_CFG_HEAP_SPACE;
   u32   heap_dc_sz     = SMX_CFG_HEAP_DC_SIZE;
   u32   stk_scan       = SMX_CFG_STACK_SCAN;
   u32   stk_pad_sz     = SMX_CFG_STACK_PAD_SIZE;
   u32   stkpl_blk_num  = SMX_CFG_NUM_STACKS;
   u32   stkpl_blk_sz   = SMX_CFG_STACK_BLK_SIZE;
   u32   stk_sz_idle    = SMX_CFG_STACK_SIZE_IDLE;
   u32   rtcb_size      = SMX_CFG_RTCB_SIZE;
   u32   rtc_frame      = SMX_CFG_RTC_FRAME;
   u32   eb_size        = (u32)__section_size("EB");
   u32   evb_size       = (u32)__section_size("EVB");
   u32   ht_size        = SMX_CFG_HT_SIZE;
   u32   pp_obj_num     = SMX_CFG_PP_OBJ_NUM;
   u32   pp_obj_size    = SMX_CFG_PP_OBJ_SIZE;
} smx_cf;

#if defined(SMXAWARE_LIVE) && !defined(SMXNS_DEMO)
#define LOCALIP    "10.1.1.100"     /* Set our IP here */
//#define LOCALIP    "192.168.0.10"   /* Set our IP here */
#define LOCALMASK  "255.255.255.0"  /* Set our subnet mask here */
#endif

#define MMO_REGIONS 0               /* Set to 1 to enable all memory regions display */

#ifdef __cplusplus
extern "C" {
#endif

BOOLEAN smxaware_live_init(void);
BOOLEAN smxaware_live_exit(void);

/* These variables must be public so smxAware can read them. */

u32      sa_ready = 0;              /* sa_ready is read only by smxAware DLL */
u32      sa_booleansz = 1;          /* sa_booleansz is read only by smxAware DLL */
u32      sa_SMX_NUM_ERRORS;         /* sa_SMX_NUM_ERRORS is read only by smxAware DLL */
SA_PRINT sa_print;
u32      sa_LiveIsInCapture = FALSE;
u32      sa_endian_test  = 0x01234567;
u32      sa_SMX_HEAP_FENCES = EH_NUM_FENCES; /* dwords above and below data block */
u32      sa_EH_NUM_HEAPS    = EH_NUM_HEAPS;

u32      sa_SMX_RSA_SIZE    = SMX_RSA_SIZE;
u32      sb_stk_fill_val    = SB_STK_FILL_VAL;
TCB_PTR* sa_smx_ct_address  = 0;
u16      smx_evbnDiagCnt;               /* simple counter for diagnostics used in xevb.h */
u32      smx_ehv_size = sizeof(EHV);
u32      smx_tcb_size = sizeof(struct TCB);
u32      smx_lcb_size = sizeof(struct LCB);
u32      smx_BuildFlags = 0;
u32      sb_CFG_OMB_SIZE = SB_CFG_OMB_SIZE;  /* Output Message Buffer size (bytes) */
u32      sb_CFG_OMQ_SIZE = SB_CFG_OMQ_SIZE;  /* Output Message Queue size (records) */
u32      smx_pri_num  = SMX_PRI_NUM;    /* now it's the number of priority levels, to support priorities going up or down */
#if SMX_CFG_PRI_UP
u32      smx_pri_up   = 1;
#else
u32      smx_pri_up   = 0;
#endif
#if SMX_CFG_MPU
#if SB_CPU_ARMM7
u32      mp_mpu_type  = MP_MPU_TYPE_ARMM7;
#elif SB_CPU_ARMM8
u32      mp_mpu_type  = MP_MPU_TYPE_ARMM8;
#endif
u32      mp_mpu_fas   = MP_MPU_FAS;
u32      mp_mpu_sz    = MP_MPU_SZ;
u32      smx_mpr_size = 0;
#endif
#if SMX_CFG_PROFILE
u32      smx_rtcb_size = 0;
#endif
#if defined(SMX_CFG_SAVE_SUSPLOC) && SMX_CFG_SAVE_SUSPLOC
u32      sa_susploc = TRUE;
#endif
#if defined(SMX_FRPORT)
u32      smx_frport = 1;            /* FreeRTOS port in use */
#endif

static void dummy_use(u8 * par);
static SCB_PTR in_sa_print;         /* protect the sa_Print() call */

void*    smx_sstop;
u32      smx_sssize;

#ifdef __cplusplus
}
#endif


#if MMO_REGIONS
/*
   USER: Modify the definitions to match your .icf file, or set them to zero.
   Providing this information is optional and is used to create a more
   accurate smxAware Memory Map Overview display.
*/
#if defined(__IAR_SYSTEMS_ICC__)
 #if defined(SB_ICF_NEW)
/* these are in .icf file */
extern char RAM_S;
extern char RAM_E;
extern char SRAM_S;
extern char SRAM_E;
char* sa_RAM_S   = &RAM_S;
char* sa_RAM_E   = &RAM_E;
char* sa_RAM0_S  = 0;
char* sa_RAM0_E  = 0;
char* sa_RAM1_S  = 0;
char* sa_RAM1_E  = 0;
char* sa_RAM2_S  = 0;
char* sa_RAM2_E  = 0;
char* sa_SRAM_S  = &SRAM_S;
char* sa_SRAM_E  = &SRAM_E;
char* sa_SRAM0_S = 0;
char* sa_SRAM0_E = 0;
char* sa_SRAM1_S = 0;
char* sa_SRAM1_E = 0;
char* sa_SRAM2_S = 0;
char* sa_SRAM2_E = 0;
char* sa_XRAM_S  = 0;
char* sa_XRAM_E  = 0;
 #else
/* these are in .icf file */
extern char __ICFEDIT_region_RAM_start__;
extern char __ICFEDIT_region_RAM_end__;
extern char __ICFEDIT_region_SRAM_start__;
extern char __ICFEDIT_region_SRAM_end__;
char* sa_RAM_S   = &__ICFEDIT_region_RAM_start__;
char* sa_RAM_E   = &__ICFEDIT_region_RAM_end__;
char* sa_RAM0_S  = 0;
char* sa_RAM0_E  = 0;
char* sa_RAM1_S  = 0;
char* sa_RAM1_E  = 0;
char* sa_RAM2_S  = 0;
char* sa_RAM2_E  = 0;
char* sa_SRAM_S  = &__ICFEDIT_region_SRAM_start__;
char* sa_SRAM_E  = &__ICFEDIT_region_SRAM_end__;
char* sa_SRAM0_S = 0;
char* sa_SRAM0_E = 0;
char* sa_SRAM1_S = 0;
char* sa_SRAM1_E = 0;
char* sa_SRAM2_S = 0;
char* sa_SRAM2_E = 0;
char* sa_XRAM_S  = 0;
char* sa_XRAM_E  = 0;
 #endif
#else
#error Define memory ranges for Memory Map Overview in smxaware.c for your compiler.
#endif
#endif /* MMO_REGIONS */


/******* smxaware_init()
* Initialize variables and buffers smxAware needs.
*****************************************************************************/

BOOLEAN smxaware_init(void)
{
   /* Do a dummy use of each of these variables to force them to be linked, since
      a clever optimizer realizes they aren't needed and strips them out otherwise.
      They are referenced by the smxAware DLL, which is not linked to the app.
   */
   dummy_use((u8 *) &smx_Processor);
   dummy_use((u8 *) &smx_Version);
   dummy_use((u8 *) &smx_htpres);
   dummy_use((u8 *) &smx_hti);
   dummy_use((u8 *) &smx_htn);
   dummy_use((u8 *) &smx_htx);
   dummy_use((u8 *) &sa_endian_test);
   dummy_use((u8 *) &sa_SMX_HEAP_FENCES);
   dummy_use((u8 *) &sa_EH_NUM_HEAPS);
   dummy_use((u8 *) &sa_SMX_RSA_SIZE);
   dummy_use((u8 *) &sb_stk_fill_val);
   dummy_use((u8 *) &sa_smx_ct_address);
   dummy_use((u8 *) &sb_ticktmr_clkhz);
   dummy_use((u8 *) &smx_bcbs);
   dummy_use((u8 *) &smx_egcbs);
   dummy_use((u8 *) &smx_eqcbs);
   dummy_use((u8 *) &smx_mcbs);
   dummy_use((u8 *) &smx_lcbs);
   dummy_use((u8 *) &smx_mucbs);
   dummy_use((u8 *) &smx_pcbs);
   dummy_use((u8 *) &smx_picbs);
   dummy_use((u8 *) &smx_scbs);
   dummy_use((u8 *) &smx_tcbs);
   dummy_use((u8 *) &smx_tmcbs);
   dummy_use((u8 *) &smx_xcbs);
   dummy_use((u8 *) &smx_pri_num);
   dummy_use((u8 *) &smx_ehv_size);
   dummy_use((u8 *) &smx_tcb_size);
   dummy_use((u8 *) &smx_lcb_size);
   dummy_use((u8 *) &smx_pri_up);
   dummy_use((u8 *) &smx_BuildFlags);
   dummy_use((u8 *) &sb_CFG_OMB_SIZE);
   dummy_use((u8 *) &sb_CFG_OMQ_SIZE);

  #if MMO_REGIONS
   dummy_use((u8 *) &sa_RAM_S);
   dummy_use((u8 *) &sa_RAM_E);
   dummy_use((u8 *) &sa_RAM0_S);
   dummy_use((u8 *) &sa_RAM0_E);
   dummy_use((u8 *) &sa_RAM1_S);
   dummy_use((u8 *) &sa_RAM1_E);
   dummy_use((u8 *) &sa_RAM2_S);
   dummy_use((u8 *) &sa_RAM2_E);
   dummy_use((u8 *) &sa_SRAM_S);
   dummy_use((u8 *) &sa_SRAM_E);
   dummy_use((u8 *) &sa_SRAM0_S);
   dummy_use((u8 *) &sa_SRAM0_E);
   dummy_use((u8 *) &sa_SRAM1_S);
   dummy_use((u8 *) &sa_SRAM1_E);
   dummy_use((u8 *) &sa_SRAM2_S);
   dummy_use((u8 *) &sa_SRAM2_E);
   dummy_use((u8 *) &sa_XRAM_S);
   dummy_use((u8 *) &sa_XRAM_E);
  #endif /* MMO_REGIONS */

  #if SMX_CFG_MPU
   dummy_use((u8 *) &mp_mpu_type);
   dummy_use((u8 *) &mp_mpu_fas);
   dummy_use((u8 *) &mp_mpu_sz);
   dummy_use((u8 *) &smx_mpr_size);
   smx_mpr_size = sizeof(MPR);
  #endif
  #if SMX_CFG_PROFILE
   dummy_use((u8 *) &smx_rtcb_size);
   smx_rtcb_size = sizeof(smx_rtcb);
  #endif
  #if defined(SMX_CFG_SAVE_SUSPLOC) && SMX_CFG_SAVE_SUSPLOC
   dummy_use((u8 *) &sa_susploc);
  #endif
  #if defined(SMX_FRPORT)
   dummy_use((u8 *) &smx_frport);
  #endif
   smx_sstop = sb_SS_GET_TOP();
   smx_sssize = sb_SS_GET_SIZE();

   /* Initialize the tracing/logging feature */
   sa_PrintInit();

   sa_booleansz = sizeof(smx_htpres);
   sa_SMX_NUM_ERRORS = SMX_NUM_ERRORS;
   sa_smx_ct_address = &smx_ct;
  #if SMX_CFG_TOKENS
   smx_BuildFlags |= SMX_CFG_TOKENS_DEFINED;    /* smx_BuildFlags contains up to 32 build settings */
  #endif

   /* smxAware is ready. smxAware changes sa_ready to 0x77.
      The special value should work fine in all cases, but if there is a
      problem, try the first setting. */
   //sa_ready = 1;         /* normal value */
   sa_ready = 0x11223344;  /* special value for attach to running app */

   return TRUE;
}


/******* sa_PrintInit()
* Initialize buffer and semaphore for user print statements.
*****************************************************************************/

void sa_PrintInit(void)
{
   int i;

   sa_print.rng_buf[SMX_CFG_SA_PRINT_RING_SIZE] = 0x77; /* fence */

   /* write a dummy first entry since algorithm in smxAware skips first */
   *(u32 *)sa_print.rng_buf = 0xFFFFFFFFL;    /* dummy timestamp */
   sa_print.rng_buf[4] = 0;                   /* dummy string */

   for (i = 5; i < SMX_CFG_SA_PRINT_RING_SIZE; i++)
      sa_print.rng_buf[i] = 0;

   sa_print.next    = 5;
   sa_print.length  = SMX_CFG_SA_PRINT_RING_SIZE;
   sa_print.version = 1;
   in_sa_print = smx_SemCreate(SMX_SEM_RSRC, 1, "in_sa_print");
}

#if SMX_CFG_MPU
/*+++++++++++++++++++++++ LIMITED SWI/SVC API (UMODE) ++++++++++++++++++++++*/
//#include "xapiu.h" -- causes SemTest() problem for sa_Print()
#endif

/******* sa_Print()
* sa_Print() puts the null terminated string into a ring buffer
* to be read by the smxAware DLL kernel aware extension.
* The string will be displayed as a line in the print window.
* It also adds a record to the Event Buffer. Before the string, it puts
* smx_etime as the timestamp. This is correlated with the Event Buffer.
*
* The smxAware code assumes strings are at least 4 non-zero characters,
* so here we pad them with spaces at the end, if necessary. If a null
* string is passed or a 0 string pointer, we put "(no string)" into
* the buffer to alert the user of a possible problem in his call.
*
* Note: You may want to use sprintf() to prepare a string for sa_Print(),
*       but sprintf() requires as much as 2KB of stack for local buffers
*       and can cause undetected stack overflow because it moves the stack
*       pointer with sub sp, nn which can skip over the pad. Also, if your
*       version of sprintf() is non-reentrant, protect it with a semaphore.
*
* Note: This calls SSRs and must not be used from an ISR.
*
*****************************************************************************/

void sa_Print(const char *string)
{
#define MIN_STRLEN 4
   u32    etime_save;
   u32    len;
   char   buf[MIN_STRLEN+1];

   if (in_sa_print == 0)
      return;   /* sa_PrintInit() has not run or failed */

   if (sa_LiveIsInCapture)
      return;   /* disable the filling of the print ring because smxAware Live is reading it */

   /* wait if another task is already in sa_Print() */
   if (!smx_SemTest(in_sa_print, SMX_TMO_NOWAIT))
   {
      if (smx_clsr)
         return;  /* skip printing message since we're in an LSR and can't wait */
      smx_SemTest(in_sa_print, SMX_TMO_INF);
   }

   /* handle short strings (see comment above this function) */
   len = strlen(string);
   if (len == 0)
      string = "(no string)";
   else if (len < MIN_STRLEN)
   {
      strcpy(buf, string);
      memset(buf+len, ' ', MIN_STRLEN-len);  /* pad with spaces */
      buf[MIN_STRLEN] = 0;
      string = buf;
   }

   /* Write smx_etime to print ring and in EVB record. Address can be on any
      boundary, so on processors that don't allow writes on byte boundaries,
      (e.g. ARM) write it one byte at a time.
   */
   etime_save = smx_SysEtimeGet();  /* save smx_etime to ensure print ring and EVB use exact same value for smx_etime */
   sb_WRITE32_UNALIGNED(&sa_print.rng_buf[(u32)(sa_print.next)], etime_save);
   smx_EVB_LOG_USER_PRINT(etime_save, sa_print.next);

   sa_print.next += 4;
   if (sa_print.next >= SMX_CFG_SA_PRINT_RING_SIZE) sa_print.next = 0;

   /* write string immediately after smx_etime */
   do
   {
      sa_print.rng_buf[(u32)(sa_print.next)] = *string;
      sa_print.next++;
      if (sa_print.next >= SMX_CFG_SA_PRINT_RING_SIZE)
         sa_print.next = 0;
   } while (*string++);  /* make sure the null is written, then exit loop */
   smx_SemSignal(in_sa_print);
}

/******* sa_PrintVals("var top = %d   bottom = %d", (long) top, (long) bottom)
* sa_PrintVals() will merge one or two values in the string at the
* position of the %d.
* If only one value is to be merged then use only one %d
*
* Note: This calls SSRs and must not be used from an ISR.
*
*****************************************************************************/

void sa_PrintVals(const char *s, long val1, long val2)
{
   #define BUF_MAX  80
   char buf[BUF_MAX+40];
   char* bufp;
   int which_val = 1;

   bufp = buf;
   while (*s != 0x00 && bufp < buf+BUF_MAX)
   {
      if (*s == '%')
      {  /* assume _ltoa() is reentrant */
         _ltoa((which_val == 1) ? val1 : val2, bufp, 10);
         for (; *bufp != '\0'; ++bufp) {}  /* find the new end of string */

         which_val++;
         s+=2;   /* skip the %d */
      }
      else
         *bufp++ = *s++;
   }
   *bufp = 0x00;  /* needed if 1 or more chars follow last %d */
   sa_Print(buf);
}

#if SMX_CFG_MPU
/*+++++++++++++++++++++++++ FULL DIRECT API (PMODE) ++++++++++++++++++++++++*/
#include "xapip.h"
#endif


/* This function is used to force reference to some variables so optimizer doesn't
   strip them out, thinking they're unused.
*/
volatile u32 dummy_count = 0;
static void dummy_use(u8 *par)
{
   dummy_count += *par;  /* do nothing */
}

#if defined(SMXFS)
/*
* smxFS smxAware interface.
*
* This section provides smxAware with constants or symbol addresses that
* are not otherwise accessible by smxAware.
*
* If smxFS does not expand or appear in the smxAware top level objects list
* here are some things to check:
*
* 1) smxFS must be 1.40 or greater
* 2) This file (smxaware.c) must be linked in.
* 3) smxaware_smxfs_init() must be called in smxfs_init() in smxmods.c
* 4) static must be removed from "PDEVICEHANDLE g_DevHandle[SFS_MAX_DEV_NUM];"  in fapi.c
* 5) Set SFS_FILENAME_IN_HANDLE to 1 in fcfg.h to see the open file names and paths
*
*****************************************************************************/
#include "smxfs.h"

/* This structure is a copy of the same structure in smxAware3A\smxAddOns\smxFS\smxFS.cpp.
   It is initialized when the target is started.
*/
typedef struct
{
   u32 data_version;
   u32 SFS_VERSION_value;
   u32 DEVICEHANDLE_size;
   u32 SFS_MAX_DEV_NUM_value;
   u32 SFS_FILENAME_IN_HANDLE_value;

   u32 SFS_FREECLUS_SUPPORT_defined;    /* used in struct DEVICEHANDLE  added in data_version = 101 */
   u32 SFS_FAT_FSINFO_SUPPORT_defined;  /* used in struct DEVICEHANDLE  added in data_version = 101 */
   u32 SFS_USE_FAT32_FSINFO_defined;    /* used in struct DEVICEHANDLE  added in data_version = 101 */

   u32 unused3;

} SA_SMXFS_CONST_TYPE;
SA_SMXFS_CONST_TYPE smxFsConst = {0};  /* Set data_version to 0. If smxAware reads this data structure and data_version is 0 then it has not yet been initialized. */

void smxaware_smxfs_init(void)
{
   smxFsConst.data_version                 = 101;   /* 101 includes SFS_FREECLUS_SUPPORT_defined */
   smxFsConst.SFS_VERSION_value            = SFS_VERSION;       /* The first version for smxAware is 0x0141 = XX.XX */
   smxFsConst.DEVICEHANDLE_size            = sfs_getdevhandlesize();
   smxFsConst.SFS_MAX_DEV_NUM_value        = SFS_MAX_DEV_NUM;
   smxFsConst.SFS_FILENAME_IN_HANDLE_value = SFS_FILENAME_IN_HANDLE;
#if SFS_FREECLUS_SUPPORT
   smxFsConst.SFS_FREECLUS_SUPPORT_defined = TRUE;
#else
   smxFsConst.SFS_FREECLUS_SUPPORT_defined = FALSE;
#endif

#if SFS_FAT_FSINFO_SUPPORT
   smxFsConst.SFS_FAT_FSINFO_SUPPORT_defined = TRUE;
#else
   smxFsConst.SFS_FAT_FSINFO_SUPPORT_defined = FALSE;
#endif

#if SFS_USE_FAT32_FSINFO
   smxFsConst.SFS_USE_FAT32_FSINFO_defined = TRUE;
#else
   smxFsConst.SFS_USE_FAT32_FSINFO_defined = FALSE;
#endif
}

#endif /* SMXFS */

#if defined(SMXNS)
/*
* smxNS smxAware interface.
*
* This section provides smxAware with constants or symbol addresses that
* are not otherwise accessible by smxAware.
*
* If smxNS does not expand or appear in the smxAware top level objects list
* here are some things to check:
*
* 1) smxNS must be 2.60 or greater
* 2) This file (smxaware.c) must be linked in.
* 3) smxaware_smxns_init() must be called in smxns_init() in smxmods.c
*
*****************************************************************************/

#define NS_DATA_VERSION 102       /* 101 includes SNS_PROTO_IPV6_defined */
#include "smxns.h"

/* This structure is a copy of the same structure in smxAware3A\smxAddOns\smxNS\smxNS.cpp.
   It is initialized when the target is started.
*/
typedef struct
{
   u32 data_version;
   u32 NS_VERSION_value;
   u32 maxbuf;
   u32 nbuffs;
   u32 messh_sz;
   u32 nconns;
   u32 nconfigs;
   u32 nnets;
   u32 mess_size;
   u32 size_of_MESS;
   u32 size_of_CONNECT;
   u32 size_of_NETCONF;
   u32 size_of_NET;

   u32 SNS_PROTO_SNMP_defined;          /* used in struct CONNECT and struct NET */
   u32 IPOPTIONS_defined;               /* used in struct CONNECT */
   u32 TCP_OPTIONS_defined;             /* used in struct CONNECT */
   u32 SNS_PROTO_ARP_defined;           /* used in struct NETCONF */
   u32 SNS_PROTO_LQRP_defined;          /* used in struct NET */
   u32 SNS_PROTO_PPP_defined;           /* used in struct NET */
   u32 SNS_PROTO_SLIP_defined;          /* used in struct NET */
   u32 DHCP_defined;                    /* used in struct NET */
   u32 SNS_CONTEXT_SWITCH_FLAG_defined; /* used in struct NET */
   u32 SNS_PROTO_IPV6_defined;          /* used in struct NET, added in data_version = 101 */

   /* new for smxNS 2.80 */
   u32 narpslots;                       /* NARPSLOTS */
   u32 NetsOffsetToIfname;
   u32 NetsOffsetToARPTable;
   u32 size_of_ARPTable;
   u32 size_of_IPAlias;                 /* sizeof(IPAlias[NALIAS]) */
   u32 FrameBufSize;                    /* added in NS_DATA_VERSION 102 */
   
   u32 unused2;
   u32 unused3;
   u32 unused4;
   u32 unused5;
   u32 unused6;
   u32 unused7;

} SA_SMXNS_CONST_TYPE;
SA_SMXNS_CONST_TYPE smxNsConst = {0};   /* Set data_version to 0. If smxAware reads this data structure and data_version is 0 then it has not yet been initialized. */

void smxaware_smxns_init(void)
{
   smxNsConst.data_version      = NS_DATA_VERSION;
   smxNsConst.NS_VERSION_value  = SNS_VERSION;       /* The first version for smxAware is 0x0260 = XX.XX */
   smxNsConst.maxbuf    = MAXBUF;
   smxNsConst.nbuffs    = NBUFFS;
   smxNsConst.messh_sz  = MESSH_SZ;
   smxNsConst.nconns    = NCONNS;
#if SNS_VERSION >= 0x0280  /* v2.80 uses the new IPv4 and IPv6 data structures where the
                              netconf struct has been removed and most of the items are now in NET. */
   struct NET netStruct;
   smxNsConst.nconfigs  = 1;
   smxNsConst.narpslots            = NARPSLOTS;
   smxNsConst.NetsOffsetToIfname   = (u32) &netStruct.ifname -  (u32)&netStruct;
   smxNsConst.NetsOffsetToARPTable = (u32) &netStruct.ARPTable -  (u32)&netStruct;
   smxNsConst.size_of_ARPTable     = sizeof(struct ARPTABENTRY);
   smxNsConst.size_of_IPAlias      = sizeof(netStruct.IPAlias[NALIAS]);
#else
   smxNsConst.nconfigs  = NCONFIGS;
   smxNsConst.size_of_NETCONF = sizeof(struct NETCONF);
#endif
   smxNsConst.nnets     = NNETS;
   smxNsConst.mess_size = (int)( MAXBUF + 4 + 2 * EXTRAPADDING ) & ~(USSBUFALIGN - 1);

   smxNsConst.size_of_MESS    = sizeof(MESS);
   smxNsConst.size_of_CONNECT = sizeof(struct CONNECT);
   smxNsConst.size_of_NET     = sizeof(struct NET);
   smxNsConst.FrameBufSize    = FRAMEBUFSIZE;
#ifdef SNS_PROTO_SNMP
   smxNsConst.SNS_PROTO_SNMP_defined = TRUE;
#else
   smxNsConst.SNS_PROTO_SNMP_defined = FALSE;
#endif

#ifdef IPOPTIONS
   smxNsConst.IPOPTIONS_defined = TRUE;
#else
   smxNsConst.IPOPTIONS_defined = FALSE;
#endif

#ifdef TCP_OPTIONS
   smxNsConst.TCP_OPTIONS_defined = TRUE;
#else
   smxNsConst.TCP_OPTIONS_defined = FALSE;
#endif

#ifdef DHCP
   smxNsConst.DHCP_defined = TRUE;
#else
   smxNsConst.DHCP_defined = FALSE;
#endif

#if SNS_PROTO_ARP
   smxNsConst.SNS_PROTO_ARP_defined = TRUE;
#else
   smxNsConst.SNS_PROTO_ARP_defined = FALSE;
#endif

#if SNS_PROTO_LQRP
   smxNsConst.SNS_PROTO_LQRP_defined = TRUE;
#else
   smxNsConst.SNS_PROTO_LQRP_defined = FALSE;
#endif

#if SNS_PROTO_PPP
   smxNsConst.SNS_PROTO_PPP_defined = TRUE;
#else
   smxNsConst.SNS_PROTO_PPP_defined = FALSE;
#endif

#if SNS_PROTO_SLIP
   smxNsConst.SNS_PROTO_SLIP_defined = TRUE;
#else
   smxNsConst.SNS_PROTO_SLIP_defined = FALSE;
#endif

#if SNS_CONTEXT_SWITCH_FLAG
   smxNsConst.SNS_CONTEXT_SWITCH_FLAG_defined = TRUE;
#else
   smxNsConst.SNS_CONTEXT_SWITCH_FLAG_defined = FALSE;
#endif

#if SNS_PROTO_IPV6
   smxNsConst.SNS_PROTO_IPV6_defined = TRUE;
#else
   smxNsConst.SNS_PROTO_IPV6_defined = FALSE;
#endif

}
#endif /* SMXNS */


#if defined(SMXUSBD) /* smxUSB Device */
/*
* smxUSB Device smxAware interface.
*
* This section provides smxAware with constants or symbol addresses that
* are not otherwise accessible by smxAware.
*
* If smxUSBD does not expand or appear in the smxAware top level objects list
* here are some things to check:
*
* 1) SUD_VERSION must be 2.30 or greater
* 2) This file (smxAware.c) must be linked in.
* 3) smxaware_smxusbd_init() must be called in smxusbd_init() in smxmods.c
*
*****************************************************************************/

#include "udcfg.h"
#include "udport.h"

/* This structure is a copy of the same structure in smxAware3A\smxAddOns\smxUSBD\smxUSBD.cpp.
   It is initialized when the target is started.
*/
typedef struct
{
   u32 data_version;
   u32 USB_VERSION_value;
   u32 SMXUSBD_defined;
   u32 SUD_HIGH_SPEED_defined;
   u32 SUD_COMPOSITE_defined;

   u32 unused0;
   u32 unused1;
   u32 unused2;
   u32 unused3;

} SA_SMXUSBD_CONST_TYPE;
SA_SMXUSBD_CONST_TYPE smxUsbDeviceConst = {0};  /* Set data_version to 0. If smxAware reads this data structure and data_version is 0 then it has not yet been initialized. */

void smxaware_smxusbd_init(void)
{
   smxUsbDeviceConst.data_version      = 100;
   smxUsbDeviceConst.USB_VERSION_value = SUD_VERSION;       /* The first version for smxAware is 0x0230 = XX.XX */
#ifdef SMXUSBD
   smxUsbDeviceConst.SMXUSBD_defined = TRUE;
#else
   smxUsbDeviceConst.SMXUSBD_defined = FALSE;
#endif

#if SUD_HIGH_SPEED
   smxUsbDeviceConst.SUD_HIGH_SPEED_defined = TRUE;
#else
   smxUsbDeviceConst.SUD_HIGH_SPEED_defined = FALSE;
#endif

#if SUD_COMPOSITE
   smxUsbDeviceConst.SUD_COMPOSITE_defined = TRUE;
#else
   smxUsbDeviceConst.SUD_COMPOSITE_defined = FALSE;
#endif

#ifndef SUD_COMPOSITE
#error  udcfg.h not included
#endif
}

#endif /* SMXUSBD */


#if defined(SMXUSBH) /* smxUSB Host */
/*
* smxUSB Host smxAware interface.
*
* This section provides smxAware with constants or symbol addresses that
* are not otherwise accessible by smxAware.
*
* If smxUSBH does not expand or appear in the smxAware top level objects list
* here are some things to check:
*
* 1) SU_VERSION must be 2.30 or greater
* 2) This file (smxAware.c) must be linked in.
* 3) smxaware_smxusbh_init() must be called in smxusb_init() in smxmods.c
*
*****************************************************************************/

#include "smxusbh.h"
#include "uport.h"

/* This structure is a copy of the same structure in smxAware3A\smxAddOns\smxUSBH\smxUSBH.cpp.
   It is initialized when the target is started.
*/
typedef struct
{
   u32 data_version;
   u32 USB_VERSION_value;
   u32 size_of_SU_DRV_DEVICEINFO_T;
   u32 SMXUSBH_defined;
   u32 SU_OTG_defined;
   u32 DeviceInfo_pConfigDesc_offset;
   u32 DeviceInfo_pChildDevice_offset;
   u32 SU_DRV_MAX_ENDPOINT_NUM_value;       /* added in data version 102 */
   u32 DeviceInfo_Desc_offset;              /* added in data version 103 for USBH V3 */
   u32 HostCtrlInfo_pRootHubDevice_offset;  /* added in data version 103 for USBH V3 */
   
   u32 unused2;
   u32 unused3;

} SA_SMXUSBH_CONST_TYPE;
SA_SMXUSBH_CONST_TYPE smxUsbHostConst = {0};  /* Set data_version to 0. If smxAware reads this data structure and data_version is 0 then it has not yet been initialized. */

void smxaware_smxusbh_init(void)
{
#if (SU_VERSION < 0x0300)
   smxUsbHostConst.data_version                       = 102;                           /* version of SA_SMXUSBH_CONST_TYPE */
#else
   smxUsbHostConst.data_version                       = 103;                           /* version of SA_SMXUSBH_CONST_TYPE */
#endif
   smxUsbHostConst.USB_VERSION_value                  = SU_VERSION;                    /* The first version for smxAware is 0x0230 = XX.XX */
   smxUsbHostConst.size_of_SU_DRV_DEVICEINFO_T        = su_GetDrvInfoSize();
   smxUsbHostConst.DeviceInfo_pConfigDesc_offset      = su_GetDrvInfoConfDescOffset(); /* save the offset to USBDeviceInfo because the structure size is different */
   smxUsbHostConst.DeviceInfo_pChildDevice_offset     = su_GetDrvInfoChildDevOffset(); /* save the offset to USBDeviceInfo because the structure size is different */
   smxUsbHostConst.SU_DRV_MAX_ENDPOINT_NUM_value      = 16;                            /* SU_DRV_MAX_ENDPOINT_NUM */
#if (SU_VERSION >= 0x0300)
   smxUsbHostConst.DeviceInfo_Desc_offset             = su_GetDrvInfoDescOffset();     /* save the offset to USBDeviceInfo.desc because the structure has changed in USBH V3 */
   smxUsbHostConst.HostCtrlInfo_pRootHubDevice_offset = su_GetRootHubDeviceOffset();   /* save the offset to SU_DRV_HOSTBUSINFO_T.pRootHubDevice because the structure has changed in USBH V3 */
#else
   smxUsbHostConst.DeviceInfo_Desc_offset             = 0;
   smxUsbHostConst.HostCtrlInfo_pRootHubDevice_offset = 0;
#endif

#ifdef SMXUSBH
   smxUsbHostConst.SMXUSBH_defined = TRUE;
#else
   smxUsbHostConst.SMXUSBH_defined = FALSE;
#endif
#if SU_OTG
   smxUsbHostConst.SU_OTG_defined = TRUE;
#else
   smxUsbHostConst.SU_OTG_defined = FALSE;
#endif
}

#endif /* SMXUSBH */


#if defined(SMXAWARE_LIVE)  /* condition out whole section if SMXAWARE_LIVE is not enabled */
#include "smxns.h"

#if defined(__GNUC__) || defined(__CC_ARM) || defined(__IAR_SYSTEMS_ICC__)
#include <string.h>     /* for memcpy() */
#else
#include <memory.h>     /* for memcpy() */
#endif

TCB_PTR smxAwareLiveTask;
void smxAwareLiveTaskMain(u32);

static u8  xferBuff[SAL_XFER_BUFF_LENGTH+4];
static u32 targetResetStatus = SAL_RECENT_RESET_OF_TARGET;
extern u32 sa_LiveIsInCapture;

static u32 ParseSmxAwareLiveCommand(smxCommandFormatStruct *commandReq, smxCommandFormatStruct  **ppCommandReply, u32 *commandReplyLength, u8 **ppDataReply, u32 *dataReplyLength);
static int WriteDataToClient(int conno, u8 *dataBuffer, u32 length, int bytesToSendMaxAtOneTime);
static void smxAwareLiveUpdateHandleTableName(void);


/******* smxaware_live_init() and exit()
*****************************************************************************/

BOOLEAN smxaware_live_init(void)
{
   smxAwareLiveTask = smx_TaskCreate(smxAwareLiveTaskMain, PRI_NORM, 1600, SMX_FL_NONE, "smxAwareLiveTask");
   if (!smxAwareLiveTask)
      return FALSE;

   if (!smx_TaskStart(smxAwareLiveTask))
      return FALSE;
   return TRUE;
}

BOOLEAN smxaware_live_exit(void)
{
   smx_TaskDelete(&smxAwareLiveTask);
   return TRUE;
}

/******* smxAwareLiveTaskMain()
* smxAware task main function.
*****************************************************************************/

void smxAwareLiveTaskMain(u32)
{
   u32* xferBuffLength;
   BOOLEAN xlateLength;
   BOOLEAN closeTheConnection;
   BOOLEAN networkReady;

   int i;
   int conno;
   int byteStatus;
   int bytesRead;
   int bytesToSendMaxAtOneTime = 0;
   u32 messageLength;                      /* this is the length of the whole message. It may span several packets */
   smxCommandFormatStruct *ppCommandReply; /* messageLength + Command + Address + Length to send back to the client */
   u32 commandReplyLength;                 /* length of commandReply to send back to the client */
   u8 *ppDataReply;                        /* data to send back to the client */
   u32 dataReplyLength;                    /* length of data to send back to the client (0 if no data) */
 #if !defined(SMXNS_DEMO)
   int errorCode;
 #endif

 #if !defined(SMXNS_DEMO)
  #if SNS_VERSION >= 0x0280
   u8 ea[Eid_SZ] = MACADDR0;
   Portcreate("enet");
   Portconfig("enet", "IP", LOCALIP);
   Portconfig("enet", "MASK", LOCALMASK);
   Portconfig("enet", "MAC", (char *)ea);
   Portconfig("enet", "LINK", "Ethernet");
   Portconfig("enet", "DRIVER", "ETHCTRL");
   errorCode = Portinit("enet", "");
  #else
   /* Set the local IP address using strings defined at top of this file */
   SetLocalIP(LOCALIP, LOCALMASK, "target", "enet");
   errorCode = Portinit("*");
  #endif
   if ( errorCode < 0 )
   {
      Nprintf(" smxAware Live failed to initialize ports due to code %d \n",errorCode);
      return;
   }
 #endif
   /* Wait for network interface to be ready before launching network apps */
   networkReady = FALSE;
   while (networkReady == FALSE)
   {
      for (i=0; i < NNETS; i++)
         if (nets[i].acd_state == IP_ADDR_ANNOUNCED)
            networkReady = TRUE;
      smx_DelayMsec(250);
   }

   xferBuffLength  = (u32*) &xferBuff[0];
   /* fill the dataValue from the symbol pointer */
   for (i = 0; i < SMX_SYMBOL_TABLE_ENTRIES; i++)
      if (smxLiveSymTable[i].symAddress > 0 && smxLiveSymTable[i].isStatic)
         smxLiveSymTable[i].dataValue = *(u32*) smxLiveSymTable[i].symAddress;

   /* Perform a passive open on port SAL_PORT_NUM
      Since the server is going to be doing a passive open, it will remain in the Nopen() function
      until the client establishes a connection.
      If the connection was successfully established, Nopen() will return a
      connection number; otherwise, it will return a negative number indicating an error.

         Nopen errors
      EHOSTUNREACH -11     Could not access the remote system.
      ETIMEDOUT    -12     Timeout.
      ECONNABORTED -14     Remote host refused the connection.
      ENOBUFS      -15     NCONNS in nscfg.h is not large enough.
   */

   while(1) /* when the connection is closed try to open another connection */
   {
      do
      {
         conno = Nopen("*", "TCP/IP", SAL_PORT_NUM, 0, 0);
         if( conno < 0 )
            smx_DelaySec(3);  /* sleep a few seconds, then try to connect again */
      } while (conno < 0);

      closeTheConnection = FALSE;
      SOCKET_RXTOUT(conno, 10000);  /* set timeout to 10 seconds */
      bytesToSendMaxAtOneTime = SOCKET_MAXDAT(conno);

      while(1)
      {
         /* The first 4 bytes are the length of the remaining packet */
         bytesRead   = 0;
         xlateLength = TRUE;
         do
         {
            /* A call to Nread() cannot return more than MAXBUF bytes at one time.
               Always ask for more than expected.
               If you ask for less than the data available you will get a EMSGSIZE error.

                  Nread errors
                              0     The remote system has closed the connection.
               ETIMEDOUT    -12     Timeout.  Read can be retried.
               ECONNABORTED -14     Protocol problem.  Normally the application should close the connection.
               EBADF        -16     The connection number is not valid.
               EWOULDBLOCK  -18     Non-blocking connection cant proceed.  Read would be retried.
               EMSGSIZE     -19     The message is too long for the supplied buffer.
            */

            byteStatus = Nread(conno, (char*) &xferBuff[bytesRead], SAL_XFER_BUFF_LENGTH);
            if (byteStatus <= 0)
            {  /* 0 = The remote system has closed the connection. */
               closeTheConnection = TRUE;
               break;
            }
            else
            {
               bytesRead += byteStatus;
               if (xlateLength && bytesRead >= 4)
               {
                  xlateLength = FALSE;
                  /* Length is the only data we convert to network byte order on the target.
                     All other data is converted by the client to save time. */
                  messageLength = htonl( *xferBuffLength);
                  if( messageLength > SAL_XFER_BUFF_LENGTH )
                     break;
               }
            }
         }
         while (bytesRead < messageLength + sizeof(messageLength));  /* messageLength does not include the 4 bytes for the message length */

         if (closeTheConnection)
            break;

         /* Parse and handle the command */
         ParseSmxAwareLiveCommand((smxCommandFormatStruct*) xferBuff, &ppCommandReply, &commandReplyLength, &ppDataReply, &dataReplyLength);

         /* if the reply data is small then append it to CommandReply
            Reuse xferBuff because we are done with the data received from smxAware Live,
            use it to send the command and data back to smxAware Live. */
         if (dataReplyLength > 0 && dataReplyLength+commandReplyLength < SAL_XFER_BUFF_LENGTH)
         {
            memcpy(xferBuff, ppCommandReply, commandReplyLength);                /* Load the CommandReply */
            memcpy(&xferBuff[commandReplyLength], ppDataReply, dataReplyLength); /* Append the response data. */
            byteStatus = WriteDataToClient(conno, (u8*) xferBuff, commandReplyLength+dataReplyLength, bytesToSendMaxAtOneTime);
         }
         else /* No copy method: first send the header in the first Nwrite(), then the data in the second Nwrite() */
         {
            /* first write the header to the Client: messageLength + command + address + length */
            byteStatus = WriteDataToClient(conno, (u8*) ppCommandReply, commandReplyLength, bytesToSendMaxAtOneTime);

            /* next write the data to the Client */
            if (dataReplyLength > 0 && byteStatus >= 0)
               byteStatus = WriteDataToClient(conno, ppDataReply, dataReplyLength, bytesToSendMaxAtOneTime);

            if (byteStatus < 0)
               break;
         }
      }
      Nclose(conno);    /* close the connection */
   }
}

/******* ParseSmxAwareLiveCommand()
* Parse commandReq (sent by the client)
* Output to be returned to the client:
*   ppCommandReply is messageLength + Command + Address + Length
*   commandReplyLength is the length of commandReply
*   ppDataReply is the data byte array of length dataReplyLength
*   dataReplyLength is the length of data (0 if no data)
*
*****************************************************************************/
smxCommandFormatStruct commandReply;

u32 ParseSmxAwareLiveCommand(smxCommandFormatStruct *commandReq, smxCommandFormatStruct  **ppCommandReply, u32 *commandReplyLength, u8 **ppDataReply, u32 *dataReplyLength)
{
   u32 dataAddress;
   static u32 first = TRUE;
   static u32 smxEvbenRestoreImage;

   if (first)
      smxEvbenRestoreImage = smx_evben;
   first = FALSE;

   commandReq->command = htonl(commandReq->command);  /* convert the command so it will be network byte order */

   commandReply.command = commandReq->command & SAL_COMMAND_MASK;
   commandReply.address = commandReq->address;
   commandReply.length  = commandReq->length;
   commandReply.command |= targetResetStatus;
#if SB_CPU_BIG_ENDIAN
   commandReply.command |= SAL_BIG_ENDIAN_TARGET;
#endif
   *commandReplyLength = SAL_DEFAULT_MESSAGE_SIZE; /* set default, sizeof messageLength + command + address + length */

   switch (commandReq->command & SAL_COMMAND_MASK)
   {
      case SAL_ALIVE_COMMAND:
         *dataReplyLength  = 0;       /* no reply data */
         targetResetStatus = 0x00;    /* turn off the SAL_RECENT_RESET_OF_TARGET bit */
         break;
      case SAL_ECHO_COMMAND:
         dataAddress      = (u32) &commandReq->data;
         *ppDataReply     = (u8*) dataAddress;    /* return the data just received */
         *dataReplyLength = commandReq->length;   /* return the length just received */
         break;
      case SAL_READ_CONST_DATA_COMMAND:
         *ppDataReply     = (u8*) &smxLiveConstDataTable;  /* return the whole smxLiveConstDataTable */
         *dataReplyLength = sizeof(smxLiveConstDataTable); /* return the length of the smxLiveConstDataTable */
         targetResetStatus = 0x00;                         /* turn off the SAL_RECENT_RESET_OF_TARGET bit */
         break;
      case SAL_READ_DATA_COMMAND:
         *ppDataReply     = (u8*) commandReq->address;     /* return the data located at the address in the command header */
         *dataReplyLength = commandReq->length;            /* number of bytes requested */
         break;
      case SAL_WRITE_DATA_COMMAND:
         dataAddress      = (u32) &commandReq->data;
         if (commandReq->address > 0x100 && commandReq->address < 0xFFFFFF00)  /* sanity check */
            memcpy((u8*) commandReq->address, (u8*) dataAddress, commandReq->length); /* move the data into the target memory */
         *dataReplyLength = 0;        /* no reply data */
         break;
      case SAL_CAPTURE_COMMAND:
         smxEvbenRestoreImage = smx_evben;
         smx_evben = 0x00;
         sa_LiveIsInCapture = TRUE;   /* stop the print buffer so it does not fill up while uploading the event buffer */
         sal_etime = smx_etime;
         smxAwareLiveUpdateHandleTableName();
         *dataReplyLength  = 0;       /* no reply data */
         break;
      case SAL_CAPTURE_DONE_COMMAND:
         smx_evben = smxEvbenRestoreImage;
         sa_LiveIsInCapture = FALSE;
         *dataReplyLength  = 0;       /* no reply data */
         break;
      default:
         commandReply.command |= SAL_UNKNOWN_COMMAND_TARGET;
         break;
   }
   commandReply.messageLength = *commandReplyLength + *dataReplyLength - sizeof(commandReply.messageLength);
   commandReply.messageLength = htonl(commandReply.messageLength);  /* convert the messageLength so it will be network byte order */
   commandReply.command       = htonl(commandReply.command);        /* convert the command so it will be network byte order */

   *ppCommandReply = &commandReply;

   return TRUE;
}

/******* WriteDataToClient()
*      Nwrite() writes a message to a connection from the specified buffer.
*      The largest buffer passed to Nwrite() should not exceed the value
*      given by the SOCKET_MAXDAT() macro.
*      For TCP connections, this will reflect the maximum segment size that is
*      indicated by the remote TCP when the connection is established.
*         Nwrite errors
*      ETIMEDOUT    -12     Timeout.  With TCP in blocking mode, this probably means the other
*                           end did not send acknowledgments as expected.  It could also mean an
*                           extremely heavy system load and that a timeout occurred before the
*                           acknowledgment could be received.  The connection should be closed.
*                           In non-blocking mode, the write should be retried.
*      ECONNABORTED -14     Protocol problem.  Normally the application should close the connection.
*      EMSGSIZE     -19     The message is too large for the internal buffer.
*      EWINZERO     -22     The peer TCP window is not large enough to accept the data. This only occurs in non-blocking mode.
*****************************************************************************/

int WriteDataToClient(int conno, u8 *dataBuffer, u32 length, int bytesToSendMaxAtOneTime)
{
   int byteStatus;
   int bytesSent;
   int bytesToSend;
   int bytesToSendThisWrite;  /* can't send more than about 1460 bytes at a time */

   bytesSent   = 0;
   bytesToSend = length;
   while (bytesSent < length) /* loop until we xmit the entire message */
   {
      bytesToSendThisWrite = (bytesToSend > bytesToSendMaxAtOneTime) ? bytesToSendMaxAtOneTime : bytesToSend;

      /* Set the TCP PSH flag. This is a hint to smxAware Live to process the incoming stream immediately.  */
      SOCKET_PUSH(conno);
      byteStatus = Nwrite(conno, (char*) &dataBuffer[bytesSent], bytesToSendThisWrite);
      if( byteStatus < 0 )
      {
        break;
      }
      else
      {
         bytesSent   += byteStatus;
         bytesToSend -= byteStatus;
      }
   }
   return byteStatus;
}

/******* smxAwareLiveUpdateHandleTableName()
* Pull the name string out and store in an array formatted for smxAware Live.
* The salHandleName[][] is in the smxLiveSymTableStruct so smxAware Live can
* read it from the target like any other symbol.
*****************************************************************************/

void smxAwareLiveUpdateHandleTableName(void)
{
   HTREC_PTR  p;
   u32 htIndex;
   for(p = smx_hti; p < smx_htn; p++)
   {
      htIndex = (p - smx_hti);
      if (htIndex >= HT_SIZE)
         return;  /* put a breakpoint here */
      memset(&salHandleName[htIndex][0], 0x00, SAL_NAME_LEN);
      if(p->h != NULL)
      {
         strncpy((char*) &salHandleName[htIndex][0],p->name, SAL_NAME_LEN);
         salHandleName[htIndex][SAL_NAME_LEN] = 0x00;
      }
   }
}

#endif /* SMXAWARE_LIVE */

#endif /* SMXAWARE */

/*
Notes:
   1. lq_size is the only field of SMX_CONF needed in smx assembly code.
      Keep it first since assembly code accesses it at _cf. Accessing it
      via _cf avoids needing to define a SMX_CONF structure in the
      assembly header for each processor and keeping it consistent.
*/
