/*
* armdefs.h                                                 Version 5.2.0
*
* ARM main definitions file. Contains:
*
* 1. Includes of the appropriate processor .h files for I/O register
*    definitions and function prototypes.
* 2. Misc processor architecture settings.
*
* Copyright (c) 2001-2022 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore
*
*****************************************************************************/

#ifndef SMX_ARMDEFS_H
#define SMX_ARMDEFS_H


/*--- Master Includes ------------------------------------------------------*/
/*
   Don't change the order of includes.

   IRQ Dispatch in Hardware or Software (ARM only not ARM-M)
   Some ARMs support hardware vectoring of interrupts. For these, our
   smx_ISR_ENTER and smx_ISR_EXIT macros are used. Otherwise, smx_irq_handler
   is hooked to the IRQ slot and it calls the IRQ dispatcher routine.
*/

#if defined(SB_CPU_AM1XXX)
#include "am1xxx.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_AM335X)
#include "am335x.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_AM35XX)
#include "am35xx.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_AT91CAP9)
#include "at91sam9.h"  /* (CAP9 is based on SAM9) */
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_AT91RM)
#include "at91rm.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_AT91SAM3)
#include "at91sam3.h"
#endif

#if defined(SB_CPU_AT91SAM4)
#include "at91sam4.h"
#endif

#if defined(SB_CPU_AT91SAM7)
#include "at91sam7.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_AT91SAM9)
#include "at91sam9.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_AT91SAMA5D2)
#include "at91sama5.h"
#include "board.h"
#include "pio.h"
#include "compiler.h"
#include "pmc.h"
#include "twid.h"
#include "act8945a.h"
#include "timer.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_AT91SAMA5D3)
#include "at91sama5.h"
#include "board.h"
#include "chip.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_AT91SAMG)
#include "samg55j19.h"
#include "board.h"
#include "ioport.h"
#include "sysclk.h"
#include "matrix.h"
#include "sleepmgr.h"
#endif

#if defined(SB_CPU_AT91SAMV)
#include "usbdefs.h"
#include "samv71q21.h"
#include "board_lowlevel.h"
#include "board.h"
#include "wdt.h"
#endif

#if defined(SB_CPU_COREMP7)
#include "coremp7.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_DA1469X)
#include "da1469x.h"
#endif

#if defined(SB_CPU_EFM32)
#include "efm32x.h"
#endif

#if defined(SB_CPU_EP93XX)
#include "ep93xx.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_F28M3X)
#include "f28m3x.h"
#endif

#if defined(SB_CPU_IMX31)
#include "imx31.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_IMX6)
#include <intrinsics.h> /* INT micro header */
#include "imx6.h"
#if !defined(SB_CPU_ARMM)
#include "drv_gic.h"
#define IRQ_DISPATCH_IN_HDW 0
#if defined(SB_CPU_IMX6SX)
#include <NXP/iMX6SX_A9.h>
#endif
#if defined(SB_CPU_IMX6UL) || defined(SB_CPU_IMX6ULL)
#include <NXP/iMX6UL.h>
#include "iMX6UL_registers.h"
#endif
#endif
#endif

#if defined(SB_CPU_IMXRT)
#include "imxrt.h"
#endif

#if defined(SB_CPU_KXX)
#include "kxx.h"
#endif

#if defined(SB_CPU_LH75401)
//#include "lh754xx_timer_driver.h"    /* conflicts with smxNet defs */
#include "lh754xx_vic_driver.h"
#include "lh75401uart.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_LH79520)
#include "lh79520bd.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_LH79524)
#include "lh79524bd.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_BRD_LOGICPD_LH7A400)
#include "abl_types.h"
#include "abl_arm922t_cp15_driver.h"
#include "lh7a400_csc.h"
#include "lh7a400_gpio.h"
#include "lh7a400_int_driver.h"
#include "lh7a400_timer_driver.h"
#include "lh7a400_uart_driver.h"
#include "sdk7a400_cpld_driver.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_BRD_LOGICPD_LH7A404)
#include "abl_types.h"
#include "abl_arm922t_cp15_driver.h"
#include "lh7a404_gpio.h"
#include "lh7a404_timer_driver.h"
#include "lh7a404_uart_driver.h"
#include "lh7a404_vic_driver.h"
#include "sdk7a404_cpld_driver.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_LMXX)
#include "hw_memmap.h"
#include "hw_types.h"
#include "sysctl.h"
#include "gpio.h"
#include "uart.h"
#endif

#if defined(SB_CPU_LPC17XX) || defined(SB_CPU_LPC40XX)
#include "lpc17xx.h"
#endif

#if defined(SB_CPU_LPC18XX)
#include "lpc18xx.h"
#endif

#if defined(SB_CPU_LPC2XXX)
#include "lpc2xxx.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_LPC23XX)
#include "lpc23xx.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_LPC24XX)
#include "lpc24xx.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_LPC288X)
#include "lpc288x.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_LPC29XX)
#include "lpc29xx.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_LPC313X) || defined(SB_CPU_LPC314X) || defined(SB_CPU_LPC315X)
#include "lpc313x.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_LPC318X)
#include "lpc318x.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_LPC32X0)
#include "lpc32x0.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_LPC43XX)
#include "lpc43xx.h"
#endif

#if defined(SB_CPU_LPC55SXX)
#include "lpc55sxx.h"
#endif

#if defined(SB_CPU_M9328MX1_MXL)
#include "mx1_def.h"
#include "mx1uart1.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_MAC71XX)
#include "iomac71xx.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_RM48)
#include "rm48.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_RM57)
#include "rm57.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_RZA1H)
#include "rza.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_S3C2XXX)
#include "s3c2xxx.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_STM32F10X)
#include "stm32f10x.h"
#endif

#if defined(SB_CPU_STM32F2XX)
#include "stm32f2xx_conf.h"
#endif

#if defined(SB_CPU_STM32F30X)
#include "stm32f30x.h"
#endif

#if defined(SB_CPU_STM32F37X)
#include "stm32f37x.h"
#endif

#if defined(SB_CPU_STM32F4XX)
 #if defined(STM32F4_HAL)
#include "stm32f4xx_hal.h"
  #if defined(SB_BRD_STMICRO_STM32F411E_DISCOVERY)
#include "stm32f411e_discovery.h"
  #elif defined(SB_BRD_STMICRO_STM32429IEVAL)
#include "stm324x9i_eval.h"
#include "stm324x9i_eval_sdram.h"
#include "stm324x9i_eval_sram.h"
  #elif defined(SB_BRD_STMICRO_STM3240GEVAL)
#include "stm324xg_eval.h"
#include "stm324xg_eval_sram.h"
  #elif defined(SB_BRD_STMICRO_STM32446EEVAL)
#include "stm32446e_eval.h"
#include "stm32446e_eval_sdram.h"
  #elif defined(SB_BRD_STMICRO_STM32469IEVAL)
#include "stm32469i_eval.h"
#include "stm32469i_eval_sdram.h"
#include "stm32469i_eval_sram.h"
  #endif
#include "uart.h"
 #elif defined(SB_CPU_STM32F42X) || defined(SB_CPU_STM32F43X)
#include "stm32f4xx_conf.h"
 #else
#include "stm32f40x_conf.h"
 #endif
#endif

#if defined(SB_CPU_STM32F7XX)
#if defined(SB_BRD_STMICRO_STM32F746G_DISCOVERY)
#include "stm32746g_discovery.h"
//#include "stm32746g_discovery_lcd.h" //MDI:DM: Commented out for partition demos
#include "stm32746g_discovery_sd.h"
#include "stm32746g_discovery_sdram.h"
#elif defined (SB_BRD_STMICRO_STM32F769IEVAL) || defined(SB_BRD_STMICRO_STM32F779IEVAL)
#include "stm32f7xx_hal.h"
#include "stm32f769i_eval.h"
#include "stm32f769i_eval_sdram.h"
#include "stm32f769i_eval_sram.h"
#include "stm32f769i_eval_io.h"
#include "stm32f769i_eval_sd.h"
#elif defined(SB_BRD_STMICRO_STM32746GEVAL) || defined(SB_BRD_EMCRAFT_STM32F746SK)
#include "stm32f7xx_hal.h"
#include "stm32756g_eval.h"
#include "stm32756g_eval_io.h"
#include "stm32756g_eval_sd.h"
#if defined(SB_BRD_STMICRO_STM32746GEVAL) 
#include "stm32f7xx_ll_fmc.h"
#include "stm32f7xx_hal_sdram.h"
#include "stm32f7xx_hal_sram.h"
#include "stm32756g_eval_sdram.h"
#include "stm32756g_eval_sram.h"
#else
#include "fmc.h"
#endif
#endif
#include "uartf7.h"
#endif

#if defined(SB_BRD_STMICRO_STM32H723ZGNUCLEO)
#include "stm32h7xx_hal.h"
#include "stm32h7xx_nucleo.h"
#include "uartf7.h"
#endif

#if defined(SB_BRD_STMICRO_STM32H753IEVAL) || defined(SB_BRD_STMICRO_STM32H743IEVAL)
#include "stm32h7xx_hal.h"
#include "stm32h743i_eval.h"
#include "stm32h743i_eval_sdram.h"
#include "stm32h743i_eval_sram.h"
#include "stm32h743i_eval_io.h"
#include "stm32h743i_eval_sd.h"
#include "uartf7.h"
#endif

#if defined(SB_BRD_STMICRO_STM32U575IEV)
#include "stm32u5xx_hal.h"
#include "stm32u575i_eval.h"
#include "stm32u575i_eval_sram.h"
#include "stm32u575i_eval_io.h"
#include "stm32u575i_eval_sd.h"
#include "uartf7.h"
#endif

#if defined(SB_CPU_STR71x)
#include "71x_lib.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_STR75X)
#include "75x_lib.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_STR91X)
#include "str91x.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_TMXX)
#include "hw_memmap.h"
#include "hw_types.h"
#include "sysctl.h"
#include "gpio.h"
#include "pin_map.h"
#include "uart.h"
#endif

#if defined(SB_CPU_TMPA9XX)
#include "tmpa9xx.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_TMS470)
#include "tms470.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_TMS570)
#include "tms570.h"
#define IRQ_DISPATCH_IN_HDW 1
#endif

#if defined(SB_CPU_VFXX)
#include "vfxx.h"
#define IRQ_DISPATCH_IN_HDW 0
#endif

#if defined(SB_CPU_XMC4XXX)
#include "xmc4xxx.h"
#endif

#if defined(SB_CPU_ZYNQ7XXX)
#include "zynq7xxx.h"
#endif

#endif /* SMX_ARMDEFS_H */
