/*
* xcfg.h                                                    Version 5.2.0
*
* smx library configuration
*
* See the SMX Quick Start for documentation of these settings.
*
* Copyright (c) 1989-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* SMX is protected by patents listed in smx.h.
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: David Moore
*
* Note: Most configuration is done in the smx_cf structure in the application.
*
*****************************************************************************/

#ifndef SMX_XCFG_H
#define SMX_XCFG_H

/* MUST MATCH SETTINGS IN <cpu_cc>.inc (e.g. xarmm_iar.inc), marked (.INC) <1> */

#define SMX_CFG_DIAGS            1  /* Enable more diagnostics. (.INC) <1> */
#define SMX_CFG_ERROR_MSGS       1
#define SMX_CFG_EVB              1  /* Enable Event Buffer */
#define SMX_CFG_HT               1
#define SMX_CFG_HT_SCAN_DUP      0
#define SMX_CFG_IDLE_RTLIM       2  /* Idle passes per runtime limit frame */
#define SMX_CFG_LOCK_NEST_LIMIT  5
#define SMX_CFG_MACROS           0  /* Use macros not functions for EVB and SSR_ENTER/EXIT */
#define SMX_CFG_MULT_SSTS        1  /* Allow multiple system service tables */

#if defined(SB_CPU_ARMM) && defined(__ICCARM__)
#define SMX_CFG_MPU              1  /* 1 for MPU support. 0 to disable. (.INC) <1> */
#else
#define SMX_CFG_MPU              0  /* Keep 0. Only ARMM IAR supported currently. */
#endif

#if SMX_CFG_MPU
#define SMX_CFG_MPU_ENABLE       1  /* Enable MPU and umode. 0 to disable temporarily (.INC) <1><2> */
#define SMX_CFG_PORTAL           0  /* Enable portals */
#define SMX_CFG_RTLIM            0  /* Enable runtime limits (.INC) <1> */
#define SMX_CFG_SSCTR            0  /* Enable system service counters */
#define SMX_CFG_TOKENS           0  /* Enable tokens */
#else
#define SMX_CFG_MPU_ENABLE       0  /* Disable MPU */
#define SMX_CFG_PORTAL           0  /* Disable portals */
#define SMX_CFG_RTLIM            0  /* Disable runtime limits */
#define SMX_CFG_SSCTR            0  /* Disable system service counters (.INC) <1> */
#define SMX_CFG_TOKENS           0  /* Disable tokens */
#endif

#define SMX_CFG_PROFILE          1  /* Enable profiling (.INC) <1> */

#ifdef SMX_CFG_PROFILE
#define SMX_CFG_RTCB_SIZE        3   /* number of runtime counter samples in smx_rtcb[][] */
#define SMX_CFG_RTC_FRAME        100 /* determines rtc frame in ticks */
#else
#define SMX_CFG_RTCB_SIZE        0
#define SMX_CFG_RTC_FRAME        0
#endif

#define SMX_CFG_SAFETY_CHECKS    1

#if defined(SB_CPU_ARM) && defined(__ICCARM__)
#define SMX_CFG_SAVE_SUSPLOC     1  /* Enable saving susploc in the current TCB. For debug. (.INC) <1> */
#else
#define SMX_CFG_SAVE_SUSPLOC     0  /* Keep 0. Only ARM and ARMM IAR supported currently */
#endif

#define SMX_CFG_STACK_SCAN       1

#if !defined(SMX_TXPORT)
#define SMX_CFG_PRI_UP           1
#define SMX_PRI_NUM              (PRI_SYS+1)  /* Must be <= 127 */
#else
#include "tx_port.h"             /* for TX_MAX_PRIORITIES */
#define SMX_CFG_PRI_UP           0
#define SMX_PRI_NUM              TX_MAX_PRIORITIES
#endif

/* Standard Priority Levels */
#if SMX_CFG_PRI_UP
typedef enum {
   PRI_MIN,
   PRI_LO,
   PRI_LO1,
   PRI_NORM,
   PRI_HI,
   PRI_HI1,
   PRI_MAX,
   PRI_SYS     /* Reserved for smx use. Do not add priorities above. */
} PRIORITIES;
#else
#define PRI_MIN   (SMX_PRI_NUM-1)
#define PRI_SYS   0
#endif

/* Safety Checks */
#if SMX_PRI_NUM > 128
#error SMX_PRI_NUM must be <= 128 in xcfg.h.
#endif

#if !SMX_CFG_MPU && (SMX_CFG_MPU_ENABLE || SMX_CFG_PORTAL)
#error Set SMX_CFG_MPU_ENABLE and SMX_CFG_PORTAL to 0 if SMX_CFG_MPU is 0 in xcfg.h.
#endif

/* Notes:
   1. Also change in the .inc file for your processor and compiler (e.g. xarmm_iar.inc).
   2. SMX_CFG_MPU_ENABLE can be set to 0 to disable use of the MPU and
      privilege levels temporarily, if they are interfering with development or
      debugging. If 0, the MPU is never enabled, all code runs privileged,
      and system calls are direct, not through SVC. This way the call stack
      is not broken as it is by an SVC call. SVC calls can be used regardless
      of this setting, by modifying the conditional in xapiu.h tagged USER:.
*/

#endif /* SMX_XCFG_H */

