/*
* mpaf7.c                                                   Version 5.2.0
*
* Default MPA and MPA templates for STM32F7xx processors.
*
* Copyright (c) 2017-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: Ralph Moore, David Moore
*
*****************************************************************************/

#include "smx.h"
#include "mpatmplt.h"

#if SMX_CFG_MPU

/* region sizes <1> */
extern u32 fscsz;
extern u32 fsdsz;
extern u32 ramsz;
extern u32 romsz;
extern u32 sramsz;
extern u32 scsz;
extern u32 sdsz;
extern u32 uccsz;
extern u32 ucdsz;

/* sections <1> <2> */
#pragma section = "EVB"
#pragma section = "fs_code"
#pragma section = "fs_data"
#pragma section = "ram_block"
#pragma section = "rom_block"
#pragma section = "sram_block"
#pragma section = "svc_code"
#pragma section = "svc_data"
#pragma section = "sys_code"
#pragma section = "sys_data"
#pragma section = "ucom_code"
#pragma section = "ucom_data"

/*===========================================================================*
                         PMODE MPA_DFLT AND TEMPLATES
*===========================================================================*/

/* Default MPA */
MPA mpa_dflt = 
{
   RGN(0 | RA("rom_block") | V, PCODE   | SRD("rom_block") | RSI("rom_block") | EN, "rom_block"),
   RGN(1 | RA("sram_block")| V, PDATARW | SRD("sram_block")| RSI("sram_block")| EN, "sram_block"),
   RGN(2 | RA("ram_block") | V, PDATARW | SRD("ram_block") | RSI("ram_block") | EN, "ram_block"),
   RGN(3 | 0x40011000      | V, IOR                        | ( 5 << 1)        | EN, "USART1"),
   RGN(4 | V, 0, 0),
   RGN(5 | V, 0, 0),
   RGN(6 | V, 0, 0),
   RGN(7 | V, 0, "stack"),
};

/* Init/Exit MPA template. Used by idle when running ainit() or aexit() */
MPA mpa_tmplt_init = 
{
   RGN(0 | RA("rom_block") | V, PCODE   | SRD("rom_block") | RSI("rom_block") | EN, "rom_block"),
   RGN(1 | RA("sram_block")| V, PDATARW | SRD("sram_block")| RSI("sram_block")| EN, "sram_block"),
   RGN(2 | RA("ram_block") | V, PDATARW | SRD("ram_block") | RSI("ram_block") | EN, "ram_block"),
   RGN(3 | 0x40000000      | V, PIOR    | RSIN(0x80000)    | EN, "IO Regs"),
   RGN(4 | V, 0, 0),
   RGN(5 | V, 0, 0),
   RGN(6 | V, 0, 0),
   RGN(7 | V, 0, "stack"),
};

/* MPA template for FatFs and ffdemo */
MPA mpa_tmplt_fs = 
{
   RGN(0 | RA("sys_code") | V, PCODE   | SRD("sys_code") | RSIC(scsz)   | EN, "sys_code"),
   RGN(1 | RA("sys_data") | V, PDATARW | SRD("sys_data") | RSIC(sdsz)   | EN, "sys_data"),
   RGN(2 | RA("fs_code")  | V, CODE    | SRD("fs_code")  | RSIC(fscsz)  | EN, "fs_code"),
   RGN(3 | RA("fs_data")  | V, DATARW  | SRD("fs_data")  | RSIC(fsdsz)  | EN, "fs_data"),
   RGN(4 | 0x40010000     | V, IOR     | N0|N1|N3|N4|N67 | (13 << 1)    | EN, "USART1, SDMMC1"), /*<3>*/
   RGN(5 | 0x40026400     | V, IOR                       | ( 9 << 1)    | EN, "DMA2"),
   RGN(6 | RA("EVB")      | V, DATARW  | SRD("EVB")      | RSI("EVB")   | EN, "EVB"),
   RGN(7 | V, 0, "stack"),
};

/*===========================================================================*
                                 UMODE TEMPLATES
*===========================================================================*/

/* MPA template for FatFs and ffdemo */
MPA mpa_tmplt_ufs = 
{
   RGN(0 | RA("ucom_code") | V, CODE    | SRD("ucom_code") | RSIC(uccsz)  | EN, "ucom_code"),
   RGN(1 | RA("ucom_data") | V, DATARW  | SRD("ucom_data") | RSIC(ucdsz)  | EN, "ucom_data"),
   RGN(2 | RA("fs_code")   | V, CODE    | SRD("fs_code")   | RSIC(fscsz) | EN, "fs_code"),
   RGN(3 | RA("fs_data")   | V, DATARW  | SRD("fs_data")   | RSIC(fsdsz) | EN, "fs_data"),
   RGN(4 | 0x40010000      | V, IOR     | N0|N1|N3|N4|N67  | (13 << 1)   | EN, "USART1, SDMMC1"), /*<3>*/
   RGN(5 | 0x40026400      | V, IOR                        | ( 9 << 1)   | EN, "DMA2"),
   RGN(6 | V, 0, 0),
   RGN(7 | V, 0, "stack"),
};

/* Notes:
   1. Exported from pdx.icf 
   2. Called "blocks" in pdx.icf, but as "sections" here.
   3. Insufficient MPU slots results in combining IO regions.
*/
#endif /* SMX_CFG_MPU */
