/*
* svc.c                                                     Version 5.2.0
*
* ARM-M SVC System call shell functions
*
* Copyright (c) 2016-2023 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* SMX is protected by patents listed in smx.h.
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Author: Ralph Moore
*
*****************************************************************************/

#include "xsmx.h"
#include "svc.h"

#if SMX_CFG_MPU

#if (defined(SMXFS) || defined(SMXUSBD)) && defined(SB_CPU_LPC55SXX)
#include "smxfs.h"
#endif

#if defined(SMXNS)
#include "smxns.h"
#endif

/*===========================================================================*
*                         System Service Jump Table <1>                      *
*===========================================================================*/

/* main system service table indices */
typeder enum {LIM, AS, ASL, BG, BM, BP, BR, BRA, BU, BPC, BPD, BPP, 
            EFP, EFS, EFT, EFTS, EGC, EGCR, EGD, EGP, EQCL, 
            EQC, EQCT, EQCR, EQD, EQS, HBP, HBS, HBSR, 
            HC, HCP, HF, HM, HP, HR, HTA, HTD, HTGH, HTGN, LIF,
            MB, MG, MM, MP, MR, MRS, MRL, MRLA, MS, MU, MXC, MXCR, MXD, MXP, 
            MUC, MUCR, MUD, MUF, MUG, MUGS, MUR, 
            PBGH, PBGP, PBM, PBRH, PBRP, 
            PIC, PICR, PID, PIG8, PIG8M, PIGP, PIGW, PIGWS, PIP8, PIP8M, PIPP, 
            PIPW, PIPWS, PIP, PIR, 
            PMGH, PMGP, PMM, PMR, PMRS, PMRL, PMRP, PMS, PMSB, 
            SC, SCR, SD, SP, SS, ST, STS, SEG, SPK, SPHC, SSG, SWI, 
            TB, TC, TCR, TD, TL, TLK, TLKC, TP, TR, TSET, TSL, TSLS, TS, TSN, 
            TSO, TSU, TU, TUQ, TY, 
            TMRD, TMRP, TMRR, TMRSP, TMRS, TMRSA, TMRSO,
           #if defined(SMXPP) 
            PPOG, PPOR, 
           #endif
           #if SMX_CFG_EVB
            ELU4, ELUP,
           #endif
            IRQC, IRQM, IRQU, PK, PTMG,
            MOC, MOV,
            MPAC, MPACL, MPASM, 
           #if SMX_CFG_PORTAL
            FPC, FPO, FPR, FPS, FTPS, POEM, POL, POR,
           #endif
           #if (defined(SMXFS) || defined(SMXUSBD)) && SFS_DRV_MMCSD
            SFS_MSDSC,
           #endif
            EM, END
           } SSNDX;

/* main system service jump table */
u32 smx_sst[] = {
   (u32)END,
   (u32)smx_SchedAutoStop,
   (u32)smx_SchedAutoStopLSR,
   (u32)smx_BlockGet,
   (u32)smx_BlockMake,
   (u32)smx_BlockPeek,
   (u32)smx_BlockRel,
   (u32)smx_BlockRelAll,
   (u32)smx_BlockUnmake,
   (u32)smx_BlockPoolCreate,
   (u32)smx_BlockPoolDelete,
   (u32)smx_BlockPoolPeek,
   (u32)smx_EventFlagsPulse,
   (u32)smx_EventFlagsSet,
   (u32)smx_EventFlagsTest,
   (u32)smx_EventFlagsTestStop,
   (u32)smx_EventGroupClear,
   (u32)smx_EventGroupCreate,
   (u32)smx_EventGroupDelete,
   (u32)smx_EventGroupPeek,
   (u32)smx_EventQueueClear,  
   (u32)smx_EventQueueCount,
   (u32)smx_EventQueueCountStop,
   (u32)smx_EventQueueCreate,
   (u32)smx_EventQueueDelete,
   (u32)smx_EventQueueSignal,
   (u32)smx_HeapBinPeek,
   (u32)smx_HeapBinScan,
   (u32)smx_HeapBinSort,
   (u32)smx_HeapCalloc,
   (u32)smx_HeapChunkPeek,
   (u32)smx_HeapFree,
   (u32)smx_HeapMalloc,
   (u32)smx_HeapPeek,
   (u32)smx_HeapRealloc,
   (u32)smx_HTAdd,
   (u32)smx_HTDelete,
   (u32)smx_HTGetHandle,
   (u32)smx_HTGetName,
   (u32)smx_LSRInvokeF,
   (u32)smx_MsgBump,
   (u32)smx_MsgGet,
   (u32)smx_MsgMake,
   (u32)smx_MsgPeek,
   (u32)smx_MsgReceive,
   (u32)smx_MsgReceiveStop,
   (u32)smx_MsgRel,
   (u32)smx_MsgRelAll,
   (u32)smx_MsgSend,
   (u32)smx_MsgUnmake,
   (u32)smx_MsgXchgClear,
   (u32)smx_MsgXchgCreate,
   (u32)smx_MsgXchgDelete,
   (u32)smx_MsgXchgPeek,
   (u32)smx_MutexClear,   
   (u32)smx_MutexCreate,
   (u32)smx_MutexDelete,
   (u32)smx_MutexFree,
   (u32)smx_MutexGet,
   (u32)smx_MutexGetStop,
   (u32)smx_MutexRel,
   (u32)smx_PBlockGetHeap, 
   (u32)smx_PBlockGetPool,
   (u32)smx_PBlockMake,
   (u32)smx_PBlockRelHeap,
   (u32)smx_PBlockRelPool,
   (u32)smx_PipeClear,
   (u32)smx_PipeCreate,
   (u32)smx_PipeDelete,
   (u32)smx_PipeGet8,
   (u32)smx_PipeGet8M,
   (u32)smx_PipeGetPkt,
   (u32)smx_PipeGetWait, 
   (u32)smx_PipeGetWaitStop,
   (u32)smx_PipePut8,
   (u32)smx_PipePut8M,
   (u32)smx_PipePutPkt,
   (u32)smx_PipePutWait,
   (u32)smx_PipePutWaitStop,
   (u32)smx_PipePeek,
   (u32)smx_PipeResume,
   (u32)smx_PMsgGetHeap,
   (u32)smx_PMsgGetPool,
   (u32)smx_PMsgMake,
   (u32)smx_PMsgReceive,
   (u32)smx_PMsgReceiveStop,
   (u32)smx_PMsgRel,
   (u32)smx_PMsgReply,
   (u32)smx_PMsgSend,
   (u32)smx_PMsgSendB,
   (u32)smx_SemClear,
   (u32)smx_SemCreate,
   (u32)smx_SemDelete,
   (u32)smx_SemPeek,
   (u32)smx_SemSignal,
   (u32)smx_SemTest,
   (u32)smx_SemTestStop,
   (u32)smx_SysEtimeGet,
   (u32)smx_SysPeek,  
   (u32)smx_SysPseudoHandleCreate,
   (u32)smx_SysStimeGet,
   (u32)smx_SysWhatIs,
   (u32)smx_TaskBump,
   (u32)smx_TaskCurrent,
   (u32)smx_TaskCreate,
   (u32)smx_TaskDelete,
   (u32)smx_TaskLocate,
   (u32)smx_TaskLock,
   (u32)smx_TaskLockClear,
   (u32)smx_TaskPeek,
   (u32)smx_TaskResume,
   (u32)smx_TaskSet,
   (u32)smx_TaskSleep,
   (u32)smx_TaskSleepStop,
   (u32)smx_TaskStart,
   (u32)smx_TaskStartNew,
   (u32)smx_TaskStop,
   (u32)smx_TaskSuspend,
   (u32)smx_TaskUnlock,
   (u32)smx_TaskUnlockQuick,
   (u32)smx_TaskYield,
   (u32)smx_TimerDup, 
   (u32)smx_TimerPeek,
   (u32)smx_TimerReset,
   (u32)smx_TimerSetPulse,
   (u32)smx_TimerStart,
   (u32)smx_TimerStartAbs,
   (u32)smx_TimerStop,
  #if defined(SMXPP)
   (u32)smxpp_ObjGet,
   (u32)smxpp_ObjRel,
  #endif
  #if SMX_CFG_EVB
   (u32)smx_EVBLogUser4,
   (u32)smx_EVBLogUserPrint,
  #endif
   (u32)sb_IRQClear,
   (u32)sb_IRQMask,
   (u32)sb_IRQUnmask,
   (u32)sb_Peek,
   (u32)sb_PtimeGet,
   (u32)sb_MsgOutConst,
   (u32)sb_MsgOutVar,
   (u32)mp_MPACreate,
   (u32)mp_MPACreateLSR,
   (u32)mp_MPASlotMove,
  #if SMX_CFG_PORTAL
   (u32)mp_FPortalClose,
   (u32)mp_FPortalOpen,
   (u32)mp_FPortalReceive,
   (u32)mp_FPortalSend,
   (u32)mp_FTPortalSend,
   (u32)mp_PortalEM,
   (u32)mp_PortalLog,
   (u32)mp_PortalRet,
  #endif
  #if (defined(SMXFS) || defined(SMXUSBD)) && SFS_DRV_MMCSD && defined(SB_CPU_LPC55SXX)
   (u32)MMCSD_SetClk,
  #endif
   (u32)smx_EM,
};
#pragma default_variable_attributes =

/*===========================================================================*
*                                uSSR Shells                                 *
*===========================================================================*/

#include "xapiu.h"

#pragma default_function_attributes = @ ".svc_text"

NI void smxu_SchedAutoStop(void)
{
   sb_SVC(AS)
}

NI void smxu_SchedAutoStopLSR(void)
{
   sb_SVC(ASL)
}

NI BCB_PTR smxu_BlockGet(PCB_PTR pool, u8 **bpp, u32 clrsz, BCB_PTR* bhp)
{
   sb_SVC(BG)
}

NI BCB_PTR smxu_BlockMake(PCB_PTR pool, u8 *bp, BCB_PTR* bhp)
{
   sb_SVC(BM)
}

NI u32 smxu_BlockPeek(BCB_PTR blk, SMX_PK_PARM par)
{
   sb_SVC(BP)
}

NI BOOLEAN smxu_BlockRel(BCB_PTR blk, u16 clrsz)
{
   sb_SVC(BR)
}

NI u32 smxu_BlockRelAll(TCB_PTR task)
{
   sb_SVC(BRA)
}

NI u8* smxu_BlockUnmake(PCB_PTR *pool, BCB_PTR blk)
{
   sb_SVC(BU)
}

NI PCB_PTR smxu_BlockPoolCreate(u8 *p, u16 num, u16 size, const char *name, PCB_PTR* php)
{
   sb_SVCG4(BPC)
}

NI u8* smxu_BlockPoolDelete(PCB_PTR* php)
{
   sb_SVC(BPD)
}

NI u32 smxu_BlockPoolPeek(PCB_PTR pool, SMX_PK_PARM par)
{
   sb_SVC(BPP)
}

NI BOOLEAN smxu_EventFlagsPulse(EGCB_PTR eg, u16 pulse_mask)
{
   sb_SVC(EFP)
}

NI BOOLEAN smxu_EventFlagsSet(EGCB_PTR eg, u16 set_mask, u16 pre_clear_mask)
{
   sb_SVC(EFS)
}

NI u32 smxu_EventFlagsTest(EGCB_PTR eg, u32 test_mask, u16 post_clear_mask, u32 timeout)
{
   sb_SVC(EFT)
}

NI void smxu_EventFlagsTestStop(EGCB_PTR eg, u32 test_mask, u16 post_clear_mask, u32 timeout)
{
   sb_SVC(EFTS)
}

NI BOOLEAN smxu_EventGroupClear(EGCB_PTR eg, u16 init_mask)
{
   sb_SVC(EGC)
}

NI EGCB_PTR smxu_EventGroupCreate(u32 init_mask, const char *name, EGCB_PTR* eghp)
{
   sb_SVC(EGCR)
}

NI BOOLEAN smxu_EventGroupDelete(EGCB_PTR* eghp)
{
   sb_SVC(EGD)
}

NI u32 smxu_EventGroupPeek(EGCB_PTR eg, SMX_PK_PARM par)
{
   sb_SVC(EGP)
}

NI BOOLEAN smxu_EventQueueClear(EQCB_PTR eq)
{
   sb_SVC(EQCL)
}

NI BOOLEAN smxu_EventQueueCount(EQCB_PTR eq, u32 count, u32 timeout)
{
   sb_SVC(EQC)
}

NI void smxu_EventQueueCountStop(EQCB_PTR eq, u32 count, u32 timeout)
{
   sb_SVC(EQCT)
}

NI EQCB_PTR smxu_EventQueueCreate(const char *name, EQCB_PTR* eqhp)
{
   sb_SVC(EQCR)
}

NI BOOLEAN smxu_EventQueueDelete(EQCB_PTR* eqhp)
{
   sb_SVC(EQD)
}

NI BOOLEAN smxu_EventQueueSignal(EQCB_PTR eq)
{
   sb_SVC(EQS)
}

NI u32 smxu_HeapBinPeek(u32 binno, EH_PK_PARM par, u32 hn)
{
   sb_SVCH(HBP)
}

NI BOOLEAN smxu_HeapBinScan(u32 binno, u32 fnum, u32 bnum, u32 hn)
{
   sb_SVCH4(HBS)
}

NI BOOLEAN smxu_HeapBinSort(u32 binno, u32 fnum, u32 hn)
{
   sb_SVCH(HBSR)
}

NI void* smxu_HeapCalloc(u32 num, u32 sz, u32 an, u32 hn)
{
   sb_SVCH4(HC)
}

NI u32 smxu_HeapChunkPeek(void* vp, EH_PK_PARM par, u32 hn)
{
   sb_SVCH(HCP)
}

NI BOOLEAN smxu_HeapFree(void *bp, u32 hn)
{
   sb_SVCH(HF)
}

NI void* smxu_HeapMalloc(u32 sz, u32 an, u32 hn)
{
   sb_SVCH(HM)
}

NI u32 smxu_HeapPeek(EH_PK_PARM par, u32 hn)
{
   sb_SVCH(HP)
}

NI void* smxu_HeapRealloc(void *cbp, u32 sz, u32 an, u32 hn)
{
   sb_SVCH4(HR)
}

NI BOOLEAN smxu_HTAdd(void *h, const char *name)
{
   sb_SVC(HTA)
}

NI BOOLEAN smxu_HTDelete(void *h)
{
   sb_SVC(HTD)
}

NI void* smxu_HTGetHandle(const char *name)
{
   sb_SVC(HTGH)
}

NI const char* smxu_HTGetName(void *h)
{
   sb_SVC(HTGN)
}

NI BOOLEAN smxu_LSRInvokeF(LSR_PTR lsr, u32 par)
{
   sb_SVC(LIF)
}

NI BOOLEAN smxu_LSRInvoke(LSR_PTR lsr, u32 par)
{
   sb_SVC(0xFF)
}

NI void smxu_LSRsOff(void)
{
   sb_SVC(0xFF)
}

NI BOOLEAN smxu_LSRsOn(void)
{
   sb_SVC(0xFF)
}

NI BOOLEAN smxu_MsgBump(MCB_PTR msg, u8 pri)
{
   sb_SVC(MB)
}

NI MCB_PTR smxu_MsgGet(PCB_PTR pool, u8 **bpp, u16 clrsz, MCB_PTR* mhp)
{
   sb_SVC(MG)
}

NI MCB_PTR smxu_MsgMake(u8 *bp, u32 bs, u8 pri, MCB_PTR* mhp)
{
   sb_SVC(MM)
}

NI u32 smxu_MsgPeek(MCB_PTR msg, SMX_PK_PARM par)
{
   sb_SVC(MP)
}

NI MCB_PTR smxu_MsgReceive(XCB_PTR xchg, u8 **bpp, u32 timeout, MCB_PTR* mhp)
{
   sb_SVC(MR)
}

NI void smxu_MsgReceiveStop(XCB_PTR xchg, u8 **bpp, u32 timeout, MCB_PTR* mhp)
{
   sb_SVC(MRS)
}

NI BOOLEAN smxu_MsgRel(MCB_PTR msg, u16 clrsz)
{
   sb_SVC(MRL)
}

NI u32 smxu_MsgRelAll(TCB_PTR task)
{
   sb_SVC(MRLA)
}

NI BOOLEAN smxu_MsgSend(MCB_PTR msg, XCB_PTR xchg, u8 pri, void *reply)
{
   sb_SVC(MS)
}

NI u8* smxu_MsgUnmake(MCB_PTR msg, u32* bsp)
{
   sb_SVC(MU)
}

NI BOOLEAN smxu_MsgXchgClear(XCB_PTR xchg)
{
   sb_SVC(MXC)
}

NI XCB_PTR smxu_MsgXchgCreate(SMX_XCHG_MODE mode, const char *name, XCB_PTR* xhp)
{
   sb_SVC(MXCR)
}

NI BOOLEAN smxu_MsgXchgDelete(XCB_PTR* xhp)
{
   sb_SVC(MXD)
}

NI u32 smxu_MsgXchgPeek(XCB_PTR xchg, SMX_PK_PARM par)
{
   sb_SVC(MXP)
}

NI BOOLEAN smxu_MutexClear(MUCB_PTR mtx)
{
   sb_SVC(MUC)
}

NI MUCB_PTR smxu_MutexCreate(u8 pi, u8 ceiling, const char *name, MUCB_PTR* muhp)
{
   sb_SVC(MUCR)
}

NI BOOLEAN smxu_MutexDelete(MUCB_PTR* muhp)
{
   sb_SVC(MUD)
}

NI BOOLEAN smxu_MutexFree(MUCB_PTR mtx)
{
   sb_SVC(MUF)
}

NI BOOLEAN smxu_MutexGet(MUCB_PTR mtx, u32 timeout)
{
   sb_SVC(MUG)
}

NI void smxu_MutexGetStop(MUCB_PTR mtx, u32 timeout)
{
   sb_SVC(MUGS)
}

NI BOOLEAN smxu_MutexRel(MUCB_PTR mtx)
{
   sb_SVC(MUR)
}

NI u8* smxu_PBlockGetHeap(u32 sz, u8 sn, u32 attr, const char* name, u32 hn)
{
   sb_SVCHG4(PBGH)
}

NI u8* smxu_PBlockGetPool(PCB_PTR pool, u8 sn, u32 attr, const char* name)
{
   sb_SVC(PBGP)
}

NI BOOLEAN smxu_PBlockMake(u8* bp, u32 sz, u8 sn, u32 attr, const char* name)
{
   sb_SVCG4(PBM)
}

NI BOOLEAN smxu_PBlockRelHeap(u8* bp, u8 sn, u32 hn)
{
   sb_SVCH(PBRH)
}

NI BOOLEAN smxu_PBlockRelPool(u8* bp, u8 sn, PCB_PTR pool, u32 clrsz)
{
   sb_SVC(PBRP)
}

NI BOOLEAN smxu_PipeClear(PICB_PTR pipe)
{
   sb_SVC(PIC)
}

NI PICB_PTR smxu_PipeCreate(void *ppb, u8 width, u16 length, const char *name, PICB_PTR* php)
{
   sb_SVCG4(PICR)
}

NI void* smxu_PipeDelete(PICB_PTR* php)
{
   sb_SVC(PID)
}

NI BOOLEAN smxu_PipeGet8(PICB_PTR pipe, u8 *bp)
{
   sb_SVC(PIG8)
}

NI u32 smxu_PipeGet8M(PICB_PTR pipe, u8 *bp, u32 lim)
{
   sb_SVC(PIG8M)
}

NI void smxu_PipeGetPkt(PICB_PTR pipe, void *pdst)
{
   sb_SVC(PIGP)
}

NI BOOLEAN smxu_PipeGetWait(PICB_PTR pipe, void *pdst, u32 timeout)
{
   sb_SVC(PIGW)
}

NI void smxu_PipeGetWaitStop(PICB_PTR pipe, void *pdst, u32 timeout)
{
   sb_SVC(PIGWS)
}

NI BOOLEAN smxu_PipePut8(PICB_PTR pipe, u8 b)
{
   sb_SVC(PIP8)
}

NI u32 smxu_PipePut8M(PICB_PTR pipe, u8 *bp, u32 lim)
{
   sb_SVC(PIP8M)
}

NI void smxu_PipePutPkt(PICB_PTR pipe, void *psrc)
{
   sb_SVC(PIPP)
}

NI BOOLEAN smxu_PipePutWait(PICB_PTR pipe, void *psrc, u32 timeout, u32 mode)
{
   sb_SVC(PIPW)
}

NI void smxu_PipePutWaitStop(PICB_PTR pipe, void *psrc, u32 timeout, u32 mode)
{
   sb_SVC(PIPWS)
}

NI BOOLEAN smxu_PipePeek(PICB_PTR pipe, SMX_PK_PARM par)
{
   sb_SVC(PIP)
}

NI BOOLEAN smxu_PipeResume(PICB_PTR pipe)
{
   sb_SVC(PIR)
}

NI MCB_PTR smxu_PMsgGetHeap(u32 sz, u8** bpp, u8 sn, u32 attr, u32 hn, MCB_PTR* mhp)
{
   sb_SVCHG4(PMGH)
}

NI MCB_PTR smxu_PMsgGetPool(PCB_PTR pool, u8** bpp, u8 sn, u32 attr, MCB_PTR* mhp)
{
   sb_SVCG4(PMGP)
}

NI MCB_PTR smxu_PMsgMake(u8* bp, u32 sz, u8 sn, u32 attr, const char* name, MCB_PTR* mhp)
{
   sb_SVCG4(PMM)
}

NI MCB_PTR smxu_PMsgReceive(XCB_PTR xchg, u8 **bpp, u8 sn, u32 timeout, MCB_PTR* mhp)
{
   sb_SVCG4(PMR)
}

NI void smxu_PMsgReceiveStop(XCB_PTR xchg, u8 **bpp, u8 sn, u32 timeout, MCB_PTR* mhp)
{
   sb_SVCG4(PMRS)
}

NI BOOLEAN smxu_PMsgRel(MCB_PTR* mhp, u16 clrsz)
{
   sb_SVCH(PMRL)
}

NI BOOLEAN smxu_PMsgReply(MCB_PTR pmsg)
{
   sb_SVC(PMRP)
}

NI BOOLEAN smxu_PMsgSend(MCB_PTR pmsg, XCB_PTR xchg, u8 pri, void *reply)
{
   sb_SVC(PMS)
}

NI BOOLEAN smxu_PMsgSendB(MCB_PTR pmsg, XCB_PTR xchg, u8 pri, void *reply)
{
   sb_SVC(PMSB)
}

NI BOOLEAN smxu_SemClear(SCB_PTR sem)
{
   sb_SVC(SC)
}

NI SCB_PTR smxu_SemCreate(SMX_SEM_MODE mode, u8 lim, const char *name, SCB_PTR* shp)
{
   sb_SVC(SCR)
}

NI BOOLEAN smxu_SemDelete(SCB_PTR* shp)
{
   sb_SVC(SD)
}

NI u32 smxu_SemPeek(SCB_PTR sem, SMX_PK_PARM par)
{
   sb_SVC(SP)
}

NI BOOLEAN smxu_SemSignal(SCB_PTR sem)
{
   sb_SVC(SS)
}

NI BOOLEAN smxu_SemTest(SCB_PTR sem, u32 timeout)
{
   sb_SVC(ST)
}

NI void smxu_SemTestStop(SCB_PTR sem, u32 timeout)
{
   sb_SVC(STS)
}

NI u32 smxu_SysEtimeGet(u32 flags)
{
   sb_SVC(SEG)
}

NI u32 smxu_SysPeek(SMX_PK_PARM par)
{
   sb_SVC(SPK)
}

NI void* smxu_SysPseudoHandleCreate(void)
{
   sb_SVC(SPHC)
}

NI u32 smxu_SysStimeGet(void)
{
   sb_SVC(SSG)
}

NI SMX_CBTYPE smxu_SysWhatIs(void *h)
{
   sb_SVC(SWI)
}

NI BOOLEAN smxu_TaskBump(TCB_PTR task, u8 pri)
{
   sb_SVC(TB)
}

NI TCB_PTR smxu_TaskCreate(FUN_PTR fun, u8 pri, u32 tlssz_ssz, u32 fl_hn, 
                                    const char *name, u8* bp, TCB_PTR* thp)
{
   sb_SVCHG4(TCR)
}

NI TCB_PTR smxu_TaskCurrent(void)
{
   sb_SVC(TC)
}

NI BOOLEAN smxu_TaskDelete(TCB_PTR* thp)
{
   sb_SVCH(TD)
}

NI void* smxu_TaskLocate(const TCB_PTR task)
{
   sb_SVC(TL)
}

NI BOOLEAN smxu_TaskLock(void)
{
   sb_SVC(TLK)
}

NI BOOLEAN smxu_TaskLockClear(void)
{
   sb_SVC(TLKC)
}

NI u32 smxu_TaskPeek(TCB_PTR task, SMX_PK_PARM par)
{
   sb_SVC(TP)
}

NI BOOLEAN smxu_TaskResume(TCB_PTR task)
{
   sb_SVC(TR)
}

NI BOOLEAN smxu_TaskSet(TCB_PTR task, SMX_ST_PARM par, u32 val1, u32 val2)
{
   sb_SVC(TSET)
}

NI BOOLEAN smxu_TaskSleep(u32 time)
{
   sb_SVC(TSL)
}

NI void smxu_TaskSleepStop(u32 time)
{
   sb_SVC(TSLS)
}

NI BOOLEAN smxu_TaskStart(TCB_PTR task, u32 par)
{
   sb_SVC(TS)
}

NI BOOLEAN smxu_TaskStartNew(TCB_PTR task, u32 par, u8 pri, FUN_PTR fun)
{
   sb_SVC(TSN)
}

NI BOOLEAN smxu_TaskStop(TCB_PTR task, u32 timeout)
{
   sb_SVC(TSO)
}

NI BOOLEAN smxu_TaskSuspend(TCB_PTR task, u32 timeout)
{
   sb_SVC(TSU)
}

NI BOOLEAN smxu_TaskUnlock(void)
{
   sb_SVC(TU)
}

NI BOOLEAN smxu_TaskUnlockQuick(void)
{
   sb_SVC(TUQ)
}

NI BOOLEAN smxu_TaskYield(void)
{
   sb_SVC(TY)
}

NI BOOLEAN smxu_TimerDup(TMCB_PTR *tmrbp, TMCB_PTR tmra, const char *name)
{
   sb_SVC(TMRD)
}

NI u32 smxu_TimerPeek(TMCB_PTR tmr, SMX_PK_PARM par)
{
   sb_SVC(TMRP)
}

NI BOOLEAN smxu_TimerReset(TMCB_PTR tmr, u32 *tlp)
{
   sb_SVC(TMRR)
}

NI BOOLEAN smxu_TimerSetPulse(TMCB_PTR tmr, u32 period, u32 width)
{
   sb_SVC(TMRSP)
}

NI BOOLEAN smxu_TimerStart(TMCB_PTR *tmhp, u32 delay, u32 period, LSR_PTR lsr, 
                                                               const char *name) 
{
   sb_SVCG4(TMRS)
}

NI BOOLEAN smxu_TimerStartAbs(TMCB_PTR *tmhp, u32 time, u32 period, LSR_PTR lsr, 
                                                               const char *name)
{
   sb_SVCG4(TMRSA)
}

NI BOOLEAN smxu_TimerStop(TMCB_PTR tmr, u32 *tlp)
{
   sb_SVC(TMRSO)
}

#if defined(SMXPP)
NI u8* smxppu_ObjGet(void)
{
   sb_SVC(PPOG)
}

NI BOOLEAN smxppu_ObjRel(u8* bp)
{
   sb_SVC(PPOR)
}
#endif

#if SMX_CFG_EVB
NI void smxu_EVBLogUser4(void *h, u32 par1, u32 par2, u32 par3, u32 par4)
{
   sb_SVCG4(ELU4)
}

NI void smxu_EVBLogUserPrint(u32 time, u32 index)
{
   sb_SVC(ELUP)
}
#endif

NI BOOLEAN sbu_IRQClear(int irq_num)
{
   sb_SVC(IRQC)
}

NI BOOLEAN sbu_IRQMask(int irq_num)
{
   sb_SVC(IRQM)
}

NI BOOLEAN sbu_IRQUnmask(int irq_num)
{
   sb_SVC(IRQU)
}

NI u32 sbu_Peek(SB_PK_PARM par)
{
   sb_SVC(PK)
}

NI u32 sbu_PtimeGet(void)
{
   sb_SVC(PTMG)
}

NI void sbu_MsgOutConst(u32 mtype, const char *mp)
{
   sb_SVC(MOC)
}

NI void sbu_MsgOutVar(u32 mtype, const char *mp)
{
   sb_SVC(MOV)
}

NI BOOLEAN mpu_MPACreate(TCB_PTR task, MPA* tmp, u32 tmsk, u32 mpasz)
{
   sb_SVC(MPAC)
}

NI BOOLEAN mpu_MPACreateLSR(LCB_PTR lsr, MPA* tmp, u32 tmsk, u32 mpasz)
{
   sb_SVC(MPACL)
}

NI BOOLEAN mpu_MPASlotMove(u8 dn, u8 sn)
{
   sb_SVC(MPASM)
}

#if SMX_CFG_PORTAL
NI BOOLEAN mpu_FPortalClose(FPCS* pch, u8 xsn)
{
   sb_SVCH(FPC)
}

NI BOOLEAN mpu_FPortalOpen(FPCS* pch, u8 csn, u32 msz, u32 nmsg, u8 pri, 
                                                u32 tmo, const char* rxname)
{
   sb_SVCHG4(FPO)
}

NI MCB_PTR mpu_FPortalReceive(FPCS* pch, u8** dpp)
{
   sb_SVC(FPR)
}

NI BOOLEAN mpu_FPortalSend(FPCS* pch, MCB* pmsg)
{
   sb_SVC(FPS)
}

NI BOOLEAN mpu_FTPortalSend(FPCS* pch, u8* bp, MCB* pmsg)
{
   sb_SVC(FTPS)
}

NI void mpu_PortalEM(PS* ph, PERRNO errnum, PERRNO* ep)
{
   sb_SVC(POEM)
}

NI void mpu_PortalLog(u32 id, u32 par1, u32 par2, u32 par3, u32 par4)
{
   sb_SVCG4(POL)
}

NI void mpu_PortalRet(u32 id, u32 rv)
{
   sb_SVC(POR)
}
#endif /* SMX_CFG_PORTAL */

#if (defined(SMXFS) || defined(SMXUSBD)) && SFS_DRV_MMCSD && defined(SB_CPU_LPC55SXX)
#ifdef __cplusplus
extern "C" {
#endif
NI void LPC313XMCI_SetClk(u32 mode)
{
    sb_SVC(SFS_MSDSC)
}
#ifdef __cplusplus
}
#endif
#endif

NI void smxu_EM(SMX_ERRNO errnum, void *h)
{
   sb_SVC(EM)
}

/* 
   Notes:
   1. When adding new services:
      a. Add unique acronym to ssndx enum, wherever convenient.
      b. Put function name into smx_sst[] in same place as ssndx.
      c. Add smxu_ shell function using above acronym.
      d. Put function prototype into xapiu.h followed by mapping macro.
      f. Add #undef for mapping macro to xapip.h.
   2. smx_IFEM is repeated by the smx_SVC_Handler in xmpu_xxx.s for all  
      indices > SMX_SST_LAST
   3. smxu_Heap calls assume that the heap mutex is free. If it is, the call
      executes and returns. If not, and if no timeout is specified, the call 
      fails and returns 0. If a timeout is specified, the current task is 
      suspended on the heap mutex. If the timeout elapses, the call fails and
      returns 0. If the mutex is gained, -2 is returned and a recall executes 
      and returns. (-2 is used because -1 is a valid return for some heap 
      functions.)
*/
#endif /* SMX_CFG_MPU */
