/*
* xsmx.h                                                    Version 5.2.0
*
* smx internal functions and macros. Include in SSRs and smx library files.
* Include smx.h in application and other library files.
*
* DO NOT INCLUDE XSMX.H IN APPLICATION CODE. INCLUDE SMX.H INSTEAD.
*
* Copyright (c) 1989-2024 Micro Digital Inc.
* All rights reserved. www.smxrtos.com
*
* SMX is protected by patents listed in smx.h.
*
* This software is confidential and proprietary to Micro Digital Inc.
* It has been furnished under a license and may be used, copied, or
* disclosed only in accordance with the terms of that license and with
* the inclusion of this header. No title to nor ownership of this
* software is hereby transferred.
*
* Authors: Alan Moore, Ralph Moore
*
*****************************************************************************/

#ifndef SMX_XSMX_H
#define SMX_XSMX_H

#include "smx.h"       /* master smx include file */

extern u32  smx_tick_ctr; /* counter used to time seconds */

/*===========================================================================*
*                        INTERNAL FUNCTION PROTOTYPES                        *
*===========================================================================*/

#ifdef __cplusplus
extern "C" {
#endif
u32      smx_BlockRelAll_F(TCB_PTR task);
void     smx_EMClear(void);
BOOLEAN  smx_EMInit(void);
void     smx_ErrorLQOvf(void);  /* reports SMXE_LQ_OVFL, for assembly files */
void     smx_EventQueueSignal_F(EQCB_PTR eq);
MCB_PTR  smx_MsgReceive_F(XCB_PTR xchg, u8** bpp, u32 timeout, MCB_PTR* mhp);
BOOLEAN  smx_MsgRel_F(MCB_PTR msg, u16 clrsz);
u32      smx_MsgRelAll_F(TCB_PTR task);
void     smx_PBlockRelSlot(u8 sn);
BOOLEAN  smx_PipeResume_F(PICB_PTR pipe);
void     smx_RelPoolStack(TCB_PTR task);
void     smx_TaskTimeout(u32 etime);
void     smx_TimerTimeout(void);

/* queuing functions */
void     smx_DQMsg(MCB_PTR m);
MCB_PTR  smx_DQFMsg(CB_PTR q);
void     smx_NQMsg(CB_PTR q, MCB_PTR m);
void     smx_PNQMsg(CB_PTR q, MCB_PTR m, u32 cb);
TCB_PTR  smx_DQFTask(CB_PTR q);
void     smx_DQRQTask(TCB_PTR t);
void     smx_DQTask(TCB_PTR t);
u32      smx_DQTimer(TMRCB_PTR tmr);
void     smx_NQRQTask(TCB_PTR t);
void     smx_NQTask(CB_PTR q, TCB_PTR t);
void     smx_NQTimer(TMRCB_PTR tmr, u32 delay);
void     smx_PNQTask(CB_PTR q, TCB_PTR t, u32 cb);
BOOLEAN  smx_ReQTask(TCB_PTR t);
BOOLEAN  smx_QTest(CB_PTR q, SMX_CBTYPE qh_type, SMX_CBTYPE qm_type);

/* control block test functions */
BOOLEAN  smx_ObjectCreateTest(u32* hp);
u32      smx_ObjectCreateTestH(u32* hp);
BOOLEAN  smx_ObjectDupTest(u32* hp);
BOOLEAN  smx_BCBTest(BCB_PTR blk, BOOLEAN priv);
BOOLEAN  smx_EGCBTest(EGCB_PTR eg, BOOLEAN priv);
BOOLEAN  smx_EQCBTest(EQCB_PTR eq, BOOLEAN priv);
BOOLEAN  smx_LCBTest(LCB_PTR lsr, BOOLEAN priv);
BOOLEAN  smx_MCBTest(MCB_PTR msg, BOOLEAN priv);
BOOLEAN  smx_MCBOnrTest(MCB_PTR msg, BOOLEAN priv);
BOOLEAN  smx_MUCBTest(MUCB_PTR mtx, BOOLEAN priv);
BOOLEAN  smx_PCBTest(PCB_PTR pool, BOOLEAN priv);
BOOLEAN  smx_PICBTest(PICB_PTR pipe, BOOLEAN priv);
BOOLEAN  smx_SCBTest(SCB_PTR sem, BOOLEAN priv);
BOOLEAN  smx_TCBTest(TCB_PTR task, BOOLEAN priv);
BOOLEAN  smx_TMRCBTest(TMRCB_PTR tmr, BOOLEAN priv);
BOOLEAN  smx_XCBTest(XCB_PTR xchg, BOOLEAN priv);

#if defined(SB_CPU_ARMM)
void     smx_MakeFrame(void);
void     smx_MakeFrameLSR(u32 par, LCB_PTR lsr);
void     smx_StartSafeLSR(u32 par);
void     smx_SchedAutoStop(void);
void     smx_SchedAutoStopLSR(void);
void     smxu_SchedAutoStop(void);
void     smxu_SchedAutoStopLSR(void);
#endif

#ifdef __cplusplus
}
#endif

/*===========================================================================*
*                               TEST MACROS                                  *
*===========================================================================*/

#define smx_TEST_BRKNQ(q) \
   ((q) != 0 && ((((q)->cbtype >= SMX_CB_RQ) && ((q)->cbtype <= SMX_CB_MTX)) || \
    ((q)->cbtype == SMX_CB_PIPE) || ((q)->cbtype == SMX_CB_EG)))

#define smx_TEST_PRIORITY(pri) \
   ((u32)(pri) < SMX_PRI_NUM || (u32)(pri) == SMX_PRI_NOCHG)

#if SMX_CFG_TOKENS
#define smx_TOKEN_TEST(task, hp, priv) smx_TokenTest(task, hp, priv)
#else
#define smx_TOKEN_TEST(task, hp, priv) TRUE
#endif

/*===========================================================================*
*                                STACK MACROS                                *
*===========================================================================*/

#if SMX_CFG_MACROS
#if SMX_CFG_STACK_SCAN
/* release stack to the scan pool to be scanned by smx_StackScan() and save 
   owner in the second word. */
#define smx_REL_POOL_STACK(task) \
   { \
      if (task->stp != NULL) \
      { \
         if (SMX_CFG_STACK_SCAN) \
         { \
            *(void**)smx_scanstack_end = (void*)(task->spp); \
            smx_scanstack_end = (void*)(task->spp); \
            *(void**)smx_scanstack_end = NULL; \
            *((u32*)smx_scanstack_end + 1) = (u32)task; \
         } \
         else \
         { \
            *(void**)(task->spp) = smx_freestack; \
            smx_freestack = (void*)(task->spp); \
         } \
         task->stp = NULL; \
      } \
   }
#else
/* release stack to the free pool */
#define smx_REL_POOL_STACK(task) \
   { \
      if (task->stp != NULL) \
      { \
         *(void**)(task->spp) = smx_freestack; \
         smx_freestack = (void*)(task->spp); \
         task->stp = NULL; \
      } \
   }
#endif
#else
#define smx_REL_POOL_STACK(task) smx_RelPoolStack(task)
#endif

#if defined(SB_CPU_ARMM) && SMX_CFG_MPU
/*
   This causes the correct return value to be passed to a task that suspended
   on a uSSR (called using the SVC handler). It is called by the complementary
   SSR that satisfies the waiting condition. For example, if a task suspends
   on smxu_SemTest() then smx_SemSignal() calls this to change the pre-set
   FALSE return to TRUE. This is a complexity that is needed when smx_SSRExit()
   runs in the SVC handler, because in that case, the smx_PENDSVH() trigger
   does not trigger there, but instead after the SVC handler exits, which
   is after the point of return(GetCTRV()). Normally, the task suspends
   right there where smx_PENDSVH() is called, and then the complementary
   SSR resumes it there, and return(GetCTRV()) gets the updated value.
*/
#define smx_PUT_RV_IN_EXR0(task) \
   { \
      if (task->sp) \
         task->flags.rv_r0 = 1; \
   }
#else
#define smx_PUT_RV_IN_EXR0(task)
#endif

/*===========================================================================*
*                               OTHER MACROS                                 *
*===========================================================================*/

/* ct may be preempted if not locked */
#define smx_DO_CTTEST() \
   { \
      if (smx_lockctr == 0 && smx_sched == 0) \
         smx_sched = SMX_CT_TEST; \
   }

#define smx_PIPE_EMPTY(p, rp) \
   ((!(p)->flags.full) && ((p)->wp >= rp) && (((p)->wp - rp) < (p)->width) ? TRUE : FALSE)

#define smx_PIPE_FULL(p, wp)  ((p)->flags.full)

/* Priority Macros */
#if SMX_CFG_PRI_UP
#define SMX_PRI_A(a, b)    (a) >  (b)   /* priority a above b */
#define SMX_PRI_AE(a, b)   (a) >= (b)   /* priority a above or equal b */
#define SMX_PRI_B(a, b)    (a) <  (b)   /* priority a below b */
#define SMX_PRI_BE(a, b)   (a) <= (b)   /* priority a below or equal b */
#define SMX_PRI_MAX(a)     (a) == (SMX_PRI_NUM-1) /* priority is maximum */
#define SMX_PRI_MIN(a)     (a) == 0               /* priority is minimum */
#else
#define SMX_PRI_A(a, b)    (a) <  (b)   /* priority a above b */
#define SMX_PRI_AE(a, b)   (a) <= (b)   /* priority a above or equal b */
#define SMX_PRI_B(a, b)    (a) >  (b)   /* priority a below b */
#define SMX_PRI_BE(a, b)   (a) >= (b)   /* priority a below or equal b */
#define SMX_PRI_MAX(a) (a) == 0               /* priority is maximum */
#define SMX_PRI_MIN(a) (a) == (SMX_PRI_NUM-1) /* priority is minimum */
#endif

#if SMX_CFG_MPU
#define smx_TASK_OP_PERMIT(task, id) \
   if (!smx_TaskOpPermit(task)) \
      smx_ERROR_EXIT(SMXE_PRIV_VIOL, FALSE, SMX_ERRH_UNS, id);
#define smx_TASK_OP_PERMIT_VAL(task, id) \
   if (!smx_TaskOpPermit(task)) \
      smx_ERROR_EXIT(SMXE_PRIV_VIOL, 0, SMX_ERRH_UNS, id);
#else
#define smx_TASK_OP_PERMIT(task, id)
#define smx_TASK_OP_PERMIT_VAL(task, id)
#endif

#endif /* SMX_XSMX_H */
